@@ -2,7 +2,7 @@
 data.cpp - Global data and resource file parsers.
  
 Copyright (C) 1998-2001 by OpenEye Scientific Software, Inc.
-Some portions Copyright (C) 2001-2006 by Geoffrey R. Hutchison
+Some portions Copyright (C) 2001-2007 by Geoffrey R. Hutchison
  
 This file is part of the Open Babel project.
 For more information, see <http://openbabel.sourceforge.net/>
@@ -21,11 +21,11 @@ GNU General Public License for more details.
 #pragma warning (disable : 4786)
 #endif
 
-#include "babelconfig.h"
-#include "data.h"
-#include "mol.h"
+#include <openbabel/babelconfig.h>
+#include <openbabel/data.h>
+#include <openbabel/mol.h>
 
-// data headers
+// data headers with default parameters
 #include "element.h"
 #include "types.h"
 #include "isotope.h"
@@ -46,7 +46,7 @@ namespace OpenBabel
   OBIsotopeTable   isotab;
   OBResidueData    resdat;
 
-  /** \class OBElementTable
+  /** \class OBElementTable data.h <openbabel/data.h>
       \brief Periodic Table of the Elements
  
       Translating element data is a common task given that many file
@@ -89,7 +89,7 @@ namespace OpenBabel
   OBElementTable::~OBElementTable()
   {
     vector<OBElement*>::iterator i;
-    for (i = _element.begin();i != _element.end();i++)
+    for (i = _element.begin();i != _element.end();++i)
       delete *i;
   }
 
@@ -198,9 +198,9 @@ namespace OpenBabel
     
     if (atomicnum < 0 || atomicnum > static_cast<int>(_element.size()))
       {
-        colors.push_back(0.0f);
-        colors.push_back(0.0f);
-        colors.push_back(0.0f);
+        colors.push_back(0.0);
+        colors.push_back(0.0);
+        colors.push_back(0.0);
         return(colors);
       }
 
@@ -305,7 +305,7 @@ namespace OpenBabel
       Init();
 
     vector<OBElement*>::iterator i;
-    for (i = _element.begin();i != _element.end();i++)
+    for (i = _element.begin();i != _element.end();++i)
       if (!strncasecmp(sym,(*i)->GetSymbol(),2))
         return((*i)->GetAtomicNum());
     if (strcasecmp(sym, "D") == 0)
@@ -323,9 +323,8 @@ namespace OpenBabel
     return(0);
   }
 
-  /** \class OBIsotopeTable
+  /** \class OBIsotopeTable data.h <openbabel/data.h>
       \brief Table of atomic isotope masses
- 
   */
 
   OBIsotopeTable::OBIsotopeTable()
@@ -376,14 +375,14 @@ namespace OpenBabel
       return 0.0;
 
     unsigned int iso;
-    for (iso = 0; iso < _isotopes[ele].size(); iso++)
+    for (iso = 0; iso < _isotopes[ele].size(); ++iso)
       if (isotope == _isotopes[ele][iso].first)
         return _isotopes[ele][iso].second;
 
     return 0.0;
   }
 
-  /** \class OBTypeTable
+  /** \class OBTypeTable data.h <openbabel/data.h>
       \brief Atom Type Translation Table
  
       Molecular file formats frequently store information about atoms in an
@@ -406,9 +405,9 @@ namespace OpenBabel
       string src,dst;
       for (atom = mol.BeginAtom(i);atom;atom = mol.EndAtom(i))
       {
-      src = atom->GetType();
-      ttab.Translate(dst,src);
-      cout << "atom number " << atom->GetIdx() << "has mol2 type " << dst << endl;
+         src = atom->GetType();
+         ttab.Translate(dst,src);
+         cout << "atom number " << atom->GetIdx() << "has mol2 type " << dst << endl;
       }
       \endcode
  
@@ -416,20 +415,23 @@ namespace OpenBabel
       - INT (Open Babel internal codes)
       - ATN (atomic numbers)
       - HYB (hybridization)
-      - MMD
+      - MMD (MacroModel)
       - MM2 (MM2 force field)
       - XYZ (element symbols from XYZ file format)
-      - ALC (Alchemy file)
-      - HAD
-      - MCML
+      - ALC (Alchemy)
+      - HAD (H added)
+      - MCML (MacMolecule)
       - C3D (Chem3D)
       - SYB (Sybyl mol2)
-      - MOL
-      - MAP
-      - DRE
+      - MOL (Sybyl mol)
+      - MAP (Gasteiger partial charge types)
+      - DRE (Dreiding)
       - XED (XED format)
       - DOK (Dock)
-      - M3D
+      - M3D (Molecular Arts M3D)
+      - IDX (Index -- i.e., line number in the file)
+      - SBN (Sybyl descriptor types for MPD files)
+      - PCM (PC Model)
   */
 
   OBTypeTable::OBTypeTable()
@@ -478,7 +480,7 @@ namespace OpenBabel
     string tmp = from;
 
     unsigned int i;
-    for (i = 0;i < _colnames.size();i++)
+    for (i = 0;i < _colnames.size();++i)
       if (tmp == _colnames[i])
         {
           _from = i;
@@ -498,7 +500,7 @@ namespace OpenBabel
     string tmp = to;
 
     unsigned int i;
-    for (i = 0;i < _colnames.size();i++)
+    for (i = 0;i < _colnames.size();++i)
       if (tmp == _colnames[i])
         {
           _to = i;
@@ -538,10 +540,10 @@ namespace OpenBabel
       return(false);
 
     if (_from >= 0 && _to >= 0 &&
-        _from < _table.size() && _to < _table.size())
+        _from < (signed)_table.size() && _to < (signed)_table.size())
       {
         vector<vector<string> >::iterator i;
-        for (i = _table.begin();i != _table.end();i++)
+        for (i = _table.begin();i != _table.end();++i)
           if ((signed)(*i).size() > _from &&  (*i)[_from] == from)
             {
               to = (*i)[_to];
@@ -555,12 +557,36 @@ namespace OpenBabel
     return(false);
   }
 
+  std::string OBTypeTable::Translate(const string &from)
+  {
+    if (!_init)
+      Init();
+
+    if (!from.size())
+      return("");
+
+    if (_from >= 0 && _to >= 0 &&
+        _from < (signed)_table.size() && _to < (signed)_table.size())
+      {
+        vector<vector<string> >::iterator i;
+        for (i = _table.begin();i != _table.end();++i)
+          if ((signed)(*i).size() > _from &&  (*i)[_from] == from)
+            {
+              return (*i)[_to];
+            }
+      }
+
+    // Throw an error, copy the string and return false
+    obErrorLog.ThrowError(__FUNCTION__, "Cannot perform atom type translation: table cannot find requested types.", obWarning);
+    return("");
+  }
+
   std::string OBTypeTable::GetFromType()
   {
     if (!_init)
       Init();
 
-    if (_from > 0 && _from < _table.size())
+    if (_from > 0 && _from < (signed)_table.size())
       return( _colnames[_from] );
     else
       return( _colnames[0] );
@@ -571,7 +597,7 @@ namespace OpenBabel
     if (!_init)
       Init();
 
-    if (_to > 0 && _to < _table.size())
+    if (_to > 0 && _to < (signed)_table.size())
       return( _colnames[_to] );
     else
       return( _colnames[0] );
@@ -580,14 +606,14 @@ namespace OpenBabel
   void Toupper(string &s)
   {
     unsigned int i;
-    for (i = 0;i < s.size();i++)
+    for (i = 0;i < s.size();++i)
       s[i] = toupper(s[i]);
   }
 
   void Tolower(string &s)
   {
     unsigned int i;
-    for (i = 0;i < s.size();i++)
+    for (i = 0;i < s.size();++i)
       s[i] = tolower(s[i]);
   }
 
@@ -609,7 +635,7 @@ namespace OpenBabel
 
     OBAtom *a1,*a2;
     OBResidue *r1,*r2;
-    vector<OBNodeBase*>::iterator i,j;
+    vector<OBAtom*>::iterator i,j;
     vector3 v;
 
     int bo;
@@ -619,6 +645,9 @@ namespace OpenBabel
     for (a1 = mol.BeginAtom(i);a1;a1 = mol.NextAtom(i))
       {
         r1 = a1->GetResidue();
+        if (r1 == NULL) // atoms may not have residues
+          continue;
+
         if (skipres && r1->GetNum() == skipres)
           continue;
 
@@ -631,6 +660,9 @@ namespace OpenBabel
         for (j=i,a2 = mol.NextAtom(j);a2;a2 = mol.NextAtom(j))
           {
             r2 = a2->GetResidue();
+            if (r2 == NULL) // atoms may not have residues
+              continue;
+
             if (r1->GetNum() != r2->GetNum())
               break;
             if (r1->GetName() != r2->GetName())
@@ -680,27 +712,28 @@ namespace OpenBabel
                 a1->SetHyb(3);
                 a1->SetFormalCharge(-1);
               }
-	    continue;
+            continue;
           }
 	
         r1 = a1->GetResidue();
+        if (r1 == NULL) continue; // atoms may not have residues
         if (skipres && r1->GetNum() == skipres)
           continue;
 
         if (r1->GetName() != rname)
           {
-	    // if SetResName fails, skip this residue
+            // if SetResName fails, skip this residue
             skipres = SetResName(r1->GetName()) ? 0 : r1->GetNum();
             rname = r1->GetName();
           }
 
-	if (LookupType(r1->GetAtomID(a1),type,hyb))
-	  {
-	    a1->SetType(type);
-	    a1->SetHyb(hyb);
-	  }
-	else // try to figure it out by bond order ???
-	  {}
+        if (LookupType(r1->GetAtomID(a1),type,hyb))
+          {
+            a1->SetType(type);
+            a1->SetHyb(hyb);
+          }
+        else // try to figure it out by bond order ???
+          {}
       }
 
     return(true);
@@ -719,30 +752,30 @@ namespace OpenBabel
     if (!vs.empty())
       {
         if (vs[0] == "BOND")
-	  {
+          {
             s = (vs[1] < vs[2]) ? vs[1] + " " + vs[2] :
-	      vs[2] + " " + vs[1];
+              vs[2] + " " + vs[1];
             bo = atoi(vs[3].c_str());
             _vtmp.push_back(pair<string,int> (s,bo));
-	  }
+          }
 
         if (vs[0] == "ATOM" && vs.size() == 4)
-	  {
+          {
             _vatmtmp.push_back(vs[1]);
             _vatmtmp.push_back(vs[2]);
             _vatmtmp.push_back(vs[3]);
-	  }
+          }
 
         if (vs[0] == "RES")
-	  _resname.push_back(vs[1]);
+          _resname.push_back(vs[1]);
 
         if (vs[0]== "END")
-	  {
+          {
             _resatoms.push_back(_vatmtmp);
             _resbonds.push_back(_vtmp);
             _vtmp.clear();
             _vatmtmp.clear();
-	  }
+          }
       }
   }
 
@@ -753,11 +786,11 @@ namespace OpenBabel
 
     unsigned int i;
 
-    for (i = 0;i < _resname.size();i++)
+    for (i = 0;i < _resname.size();++i)
       if (_resname[i] == s)
         {
-	  _resnum = i;
-	  return(true);
+          _resnum = i;
+          return(true);
         }
 
     _resnum = -1;
@@ -770,9 +803,9 @@ namespace OpenBabel
       return(0);
 
     unsigned int i;
-    for (i = 0;i < _resbonds[_resnum].size();i++)
+    for (i = 0;i < _resbonds[_resnum].size();++i)
       if (_resbonds[_resnum][i].first == s)
-	return(_resbonds[_resnum][i].second);
+        return(_resbonds[_resnum][i].second);
 
     return(0);
   }
@@ -786,9 +819,9 @@ namespace OpenBabel
     s = (s1 < s2) ? s1 + " " + s2 : s2 + " " + s1;
 
     unsigned int i;
-    for (i = 0;i < _resbonds[_resnum].size();i++)
+    for (i = 0;i < _resbonds[_resnum].size();++i)
       if (_resbonds[_resnum][i].first == s)
-	return(_resbonds[_resnum][i].second);
+        return(_resbonds[_resnum][i].second);
 
     return(0);
   }
@@ -804,11 +837,11 @@ namespace OpenBabel
     for (i = _resatoms[_resnum].begin();i != _resatoms[_resnum].end();i+=3)
       if (atmid == *i)
         {
-	  i++;
-	  type = *i;
-	  i++;
-	  hyb = atoi((*i).c_str());
-	  return(true);
+          i++;
+          type = *i;
+          i++;
+          hyb = atoi((*i).c_str());
+          return(true);
         }
 
     return(false);
@@ -820,102 +853,118 @@ namespace OpenBabel
       return;
     _init = true;
 
-    string buffer, subbuffer;
-    ifstream ifs1, ifs2, ifs3, ifs4, *ifsP;
-    // First, look for an environment variable
-    if (getenv(_envvar.c_str()) != NULL)
-      {
-        buffer = getenv(_envvar.c_str());
-	buffer += FILE_SEP_CHAR;
-
-        if (!_subdir.empty())
-	  {
-            subbuffer = buffer;
-            subbuffer += _subdir;
-            subbuffer += FILE_SEP_CHAR;
-	  }
-
-        buffer += _filename;
-        subbuffer += _filename;
-
-        ifs1.open(subbuffer.c_str());
-        ifsP= &ifs1;
-        if (!(*ifsP))
-	  {
-            ifs2.open(buffer.c_str());
-            ifsP = &ifs2;
-	  }
-      }
-    // Then, check the configured data directory
-    else // if (!(*ifsP))
-      {
-        buffer = _dir;
-        buffer += FILE_SEP_CHAR;
-
-	subbuffer = buffer;
-	subbuffer += BABEL_VERSION;
-	subbuffer += FILE_SEP_CHAR;
-
-        subbuffer += _filename;
-	buffer += _filename;
-
-        ifs3.open(subbuffer.c_str());
-        ifsP= &ifs3;
-        if (!(*ifsP))
-	  {
-            ifs4.open(buffer.c_str());
-            ifsP = &ifs4;
-	  }
-      }
-
+    ifstream ifs;
     char charBuffer[BUFF_SIZE];
-    if ((*ifsP))
+
+    OpenDatafile(ifs, _filename, _envvar);
+     
+    if ((ifs))
       {
-        while(ifsP->getline(charBuffer,BUFF_SIZE))
-	  ParseLine(charBuffer);
+        while(ifs.getline(charBuffer,BUFF_SIZE))
+          ParseLine(charBuffer);
       }
 
     else
       // If all else fails, use the compiled in values
       if (_dataptr)
         {
-	  const char *p1,*p2;
-	  for (p1 = p2 = _dataptr;*p2 != '\0';p2++)
-	    if (*p2 == '\n')
-	      {
-		strncpy(charBuffer, p1, (p2 - p1));
-		charBuffer[(p2 - p1)] = '\0';
-		ParseLine(charBuffer);
-		p1 = ++p2;
-	      }
+          const char *p1,*p2;
+          for (p1 = p2 = _dataptr;*p2 != '\0';++p2)
+            if (*p2 == '\n')
+              {
+                strncpy(charBuffer, p1, (p2 - p1));
+                charBuffer[(p2 - p1)] = '\0';
+                ParseLine(charBuffer);
+                p1 = ++p2;
+              }
         }
       else
         {
-	  string s = "Unable to open data file '";
-	  s += _filename;
-	  s += "'";
-	  obErrorLog.ThrowError(__FUNCTION__, s, obWarning);
+          string s = "Unable to open data file '";
+          s += _filename;
+          s += "'";
+          obErrorLog.ThrowError(__FUNCTION__, s, obWarning);
         }
 
-    if (ifs1)
-      ifs1.close();
-    if (ifs2)
-      ifs2.close();
-    if (ifs3)
-      ifs3.close();
-    if (ifs4)
-      ifs4.close();
+    if (ifs)
+      ifs.close();
 
     if (GetSize() == 0)
       {
-	string s = "Cannot initialize database '";
-	s += _filename;
-	s += "' which may cause further errors.";
-	obErrorLog.ThrowError(__FUNCTION__, "Cannot initialize database", obWarning);
+        string s = "Cannot initialize database '";
+        s += _filename;
+        s += "' which may cause further errors.";
+        obErrorLog.ThrowError(__FUNCTION__, "Cannot initialize database", obWarning);
       }
       
   }
 
+  /** Opens the filestream with the first file called @p filename
+     found by looking 
+     successively in the following directories:
+     - the current directory
+     - in a subdirectory (of the directory below) with the version of 
+     OpenBabel as its name
+     - the parent directory specified by the environment variable 
+     named @p envvar 
+     or "BABEL_DATADIR" if @p envvar is not specified, or the compiled-in
+     macro BABEL_DATADIR if the environment variable is not set
+
+     \param ifs        Stream to load
+     \param filename   Name of the data file to load
+     \param envvar     Name of the environment variable 
+
+     \return the full name of the file that was opened (i.e with path) except 
+     if it is in current directory
+
+  **/
+  std::string OpenDatafile(std::ifstream& ifs, const std::string& filename, 
+                           const std::string& envvar)
+  {
+    ios_base::openmode imode = ios_base::in;
+#ifdef ALL_READS_BINARY //Makes unix files compatible with VC++6
+		imode = ios_base::in|ios_base::binary;
+#endif
+
+    // check the current directory
+    ifs.close();
+    ifs.clear();
+    ifs.open(filename.c_str(),imode);
+    if(ifs)
+      return filename;
+
+    string file;
+    char* datadir = getenv(envvar.c_str());
+    if(!datadir)
+      datadir = BABEL_DATADIR;
+
+    // check the subdirectory for this version number
+    file = datadir;
+    file += FILE_SEP_CHAR;
+    file += BABEL_VERSION;
+    file += FILE_SEP_CHAR + filename;
+
+    ifs.clear();
+    ifs.open(file.c_str(),imode);
+    if(ifs)
+      return file;
+
+    // couldn't find it with the version built in, so try the parent
+    file = datadir;
+    file += FILE_SEP_CHAR;
+    file += filename;
+
+    ifs.clear();
+    ifs.open(file.c_str(),imode);
+
+    if (ifs)
+      return file;
+
+    ifs.clear();
+    ifs.close();
+    return(""); // error
+  }
+
 } // end namespace OpenBabel
 
 //! \file data.cpp
