@@ -2,7 +2,7 @@
 dlhandler_unix.cpp - Dynamic loader for UNIX (handles file format shared obj.)
 
 Copyright (C) 2004 by Chris Morley
-Some portions Copyright (C) 2004-2005 by Geoffrey R. Hutchison
+Some portions Copyright (C) 2004-2006 by Geoffrey R. Hutchison
  
 This file is part of the Open Babel project.
 For more information, see <http://openbabel.sourceforge.net/>
@@ -17,21 +17,18 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 ***********************************************************************/
 
-#include "dlhandler.h"
-#include "babelconfig.h"
+#include <openbabel/dlhandler.h>
+#include <openbabel/babelconfig.h>
 
 #include <unistd.h>
 #include <dirent.h>
-#include <stdio.h>
 #include <sys/stat.h>
 #include <dlfcn.h>
 
-#include <iostream>
-
 using namespace std;
 
 namespace OpenBabel {
-OBAPI bool tokenize(vector<string> &, const char *, const char *);
+  OBAPI bool tokenize(vector<string> &, const char *, const char *);
 }
 
 #ifndef BUFF_SIZE
@@ -68,6 +65,9 @@ int DLHandler::findFiles (std::vector <std::string>& file_list,const std::string
   
   if (getenv("BABEL_LIBDIR") != NULL)
     {
+      // environment variable should override built-in path
+      paths.clear();
+      
       strncpy(buffer,getenv("BABEL_LIBDIR"), BUFF_SIZE - 1);
       // add a trailing NULL just in case
       buffer[BUFF_SIZE - 1] = '\0';
@@ -75,27 +75,27 @@ int DLHandler::findFiles (std::vector <std::string>& file_list,const std::string
       OpenBabel::tokenize(vs, buffer, "\r\n\t :");
       
       if (vs.size() > 0)
-	{
-	  for (unsigned int i = 0; i < vs.size(); i++)
-	    paths.push_back(vs[i]);
-	}
+        {
+          for (unsigned int i = 0; i < vs.size(); ++i)
+            paths.push_back(vs[i]);
+        }
     }
   
   if (paths.size() == 0)
     paths.push_back("./"); // defaults to current directory
-
+  
   /* Old method using scandir. Replaced with readdir (below) as for example
    * Solaris pre 10 doesn't implement scandir.
    
    struct dirent **entries_pp;
    int count;
    
-   for (unsigned int i = 0; i < paths.size(); i++)
+   for (unsigned int i = 0; i < paths.size(); ++i)
    {
    currentPath = paths[i];
    count = scandir (currentPath.c_str(), &entries_pp, SCANDIR_T matchFiles, NULL);
    
-   for(int i=0; i<count; i++)
+   for(int i=0; i<count; ++i)
    {
    file_list.push_back(currentPath + getSeparator() + (entries_pp[i])->d_name);
    free(entries_pp[i]);
@@ -104,27 +104,27 @@ int DLHandler::findFiles (std::vector <std::string>& file_list,const std::string
    
    if (entries_pp)
    free(entries_pp);
-   * 
+   *
    */
   
   DIR *dp;
   struct dirent *entry;
   
-  for (unsigned int i = 0; i < paths.size(); i++)
+  for (unsigned int i = 0; i < paths.size(); ++i)
     {
       currentPath=paths[i];
       
       if ((dp = opendir(currentPath.c_str())) == NULL)
-	continue; // no big deal, this path causes an error
-      else 
+        continue; // no big deal, this path causes an error
+      else
         {
-          while((entry = readdir(dp)) != NULL) 
-	    {
-	      if (matchFiles(entry) != 0) 
-		file_list.push_back(currentPath + getSeparator() + (entry)->d_name);
-	    }
-	  closedir(dp); // calls free(dp) -- no memory leak
-	}
+          while((entry = readdir(dp)) != NULL)
+            {
+              if (matchFiles(entry) != 0)
+                file_list.push_back(currentPath + getSeparator() + (entry)->d_name);
+            }
+          closedir(dp); // calls free(dp) -- no memory leak
+        }
     }
   
   if (file_list.size() == 0)
@@ -132,9 +132,24 @@ int DLHandler::findFiles (std::vector <std::string>& file_list,const std::string
   return file_list.size();
 }
 
+int DLHandler::findFiles (std::vector<std::string>& file_list,const std::string &filename)
+{
+  if(filename.find_first_of("*?")==string::npos)
+    {
+      //no wildcard in filename
+      file_list.push_back(filename);
+      return -1;
+    }
+  size_t pos = filename.find_last_of("\\/");
+  if(pos!=string::npos)
+    return findFiles(file_list,filename.substr(pos+1), filename.substr(0,pos+1));
+  else
+    return findFiles(file_list,filename, "");
+}
+
 bool DLHandler::openLib(const string& lib_name)
 {
-  return dlopen(lib_name.c_str(), RTLD_LAZY) != 0;
+  return dlopen(lib_name.c_str(), RTLD_LAZY | RTLD_GLOBAL) != 0;
 }
 
 const char* DLHandler::getFormatFilePattern()
