@@ -1,6 +1,6 @@
 /**********************************************************************
 Copyright (C) 1998-2001 by OpenEye Scientific Software, Inc.
-Some portions Copyright (C) 2001-2005 by Geoffrey R. Hutchison
+Some portions Copyright (C) 2001-2006 by Geoffrey R. Hutchison
 Some portions Copyright (C) 2004 by Chris Morley
  
 This program is free software; you can redistribute it and/or modify
@@ -12,38 +12,41 @@ but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 ***********************************************************************/
-
-#include "mol.h"
-#include "obconversion.h"
+#include <openbabel/babelconfig.h>
+#include <openbabel/obmolecformat.h>
 
 using namespace std;
 namespace OpenBabel
 {
 
-class CacheFormat : public OBFormat
-{
-public:
+  class CacheFormat : public OBMoleculeFormat
+  {
+  public:
     //Register this format type ID
     CacheFormat()
     {
-        OBConversion::RegisterFormat("cache",this);
+      OBConversion::RegisterFormat("cac",this);
+      OBConversion::RegisterFormat("cache",this, "chemical/x-alchemy");
     }
 
     virtual const char* Description() //required
     {
-        return
-            "CAChe MolStruct format\n \
+      return
+        "CAChe MolStruct format\n \
             No comments yet\n";
     };
 
-  virtual const char* SpecificationURL()
-  {return "";}; //optional
+    virtual const char* SpecificationURL()
+    {return "";}; //optional
+
+    virtual const char* GetMIMEType() 
+    { return "chemical/x-cache"; };
 
     //Flags() can return be any the following combined by | or be omitted if none apply
     // NOTREADABLE  READONEONLY  NOTWRITABLE  WRITEONEONLY
     virtual unsigned int Flags()
     {
-        return NOTREADABLE;
+      return NOTREADABLE;
     };
 
     ////////////////////////////////////////////////////
@@ -51,38 +54,18 @@ public:
     virtual bool WriteMolecule(OBBase* pOb, OBConversion* pConv);
 
     ////////////////////////////////////////////////////
-    /// The "Convert" interface functions
-    virtual bool WriteChemObject(OBConversion* pConv)
-    {
-        //Retrieve the target OBMol
-        OBBase* pOb = pConv->GetChemObject();
-        OBMol* pmol = dynamic_cast<OBMol*> (pOb);
-        bool ret=false;
-        if(pmol)
-            ret=WriteMolecule(pmol,pConv);
+  };
 
-        std::string auditMsg = "OpenBabel::Write molecule ";
-        std::string description(Description());
-        auditMsg += description.substr( 0, description.find('\n') );
-        obErrorLog.ThrowError(__FUNCTION__,
-                              auditMsg,
-                              obAuditMsg);
+  //Make an instance of the format class
+  CacheFormat theCacheFormat;
 
-        delete pOb;
-        return ret;
-    };
-};
+  ////////////////////////////////////////////////////////////////
 
-//Make an instance of the format class
-CacheFormat theCacheFormat;
-
-////////////////////////////////////////////////////////////////
-
-bool CacheFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
-{
+  bool CacheFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
+  {
     OBMol* pmol = dynamic_cast<OBMol*>(pOb);
     if(pmol==NULL)
-        return false;
+      return false;
 
     //Define some references so we can use the old parameter names
     ostream &ofs = *pConv->GetOutStream();
@@ -111,9 +94,9 @@ bool CacheFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
     ofs << "ID xyz_coordinates             anum sym	chrg rflag\n";
 
     OBAtom *atom;
-    vector<OBNodeBase*>::iterator i;
+    vector<OBAtom*>::iterator i;
     for(atom = mol.BeginAtom(i);atom;atom = mol.NextAtom(i))
-    {
+      {
         // 16 = sizeof(type_name)
         strncpy(type_name,etab.GetSymbol(atom->GetAtomicNum()), 16);
         // sizeof(type_name) - 1)
@@ -128,7 +111,7 @@ bool CacheFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
                 type_name,
                 atom->GetFormalCharge());
         ofs << buffer << endl;
-    }
+      }
 
     ofs << "property_flags:\n";
     ofs << "object_class bond\n";
@@ -139,27 +122,27 @@ bool CacheFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
 
     char bstr[16];
     OBBond *bond;
-    vector<OBEdgeBase*>::iterator j;
+    vector<OBBond*>::iterator j;
     for (bond = mol.BeginBond(j);bond;bond = mol.NextBond(j))
-    {
+      {
         switch (bond->GetBO())
-        {
-        case 1:
+          {
+          case 1:
             strcpy(bstr,"single");
             break;
-        case 2:
+          case 2:
             strcpy(bstr,"double");
             break;
-        case 3:
+          case 3:
             strcpy(bstr,"triple");
             break;
-        default:
+          default:
             strcpy(bstr,"weak");
-        }
+          }
 
         snprintf(buffer, BUFF_SIZE, "%3d 0x7005 %s\n", bond->GetIdx()+1,bstr);
         ofs << buffer;
-    }
+      }
 
     ofs << "property_flags:\n";
     ofs << "object_class connector\n";
@@ -173,17 +156,17 @@ bool CacheFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
 
     int k;
     for (bond = mol.BeginBond(j),k=1;bond;bond = mol.NextBond(j))
-    {
+      {
         snprintf(buffer, BUFF_SIZE, "%3d 0xa1 atom bond %d %d\n",
                 k++,bond->GetBeginAtomIdx(),bond->GetIdx()+1);
         ofs << buffer;
         snprintf(buffer, BUFF_SIZE, "%3d 0xa1 atom bond %d %d\n",
                 k++,bond->GetEndAtomIdx(),bond->GetIdx()+1);
         ofs << buffer;
-    }
+      }
 
     ofs << "property_flags:\n";
     return(true);
-}
+  }
 
 } //namespace OpenBabel
