@@ -12,10 +12,9 @@ but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 ***********************************************************************/
+#include <openbabel/babelconfig.h>
 
-#include "mol.h"
-#include "obconversion.h"
-#include "obmolecformat.h"
+#include <openbabel/obmolecformat.h>
 
 using namespace std;
 namespace OpenBabel
@@ -27,6 +26,9 @@ namespace OpenBabel
     //Register this format type ID
     GaussianOutputFormat()
     {
+      OBConversion::RegisterFormat("gal",this, "chemical/x-gaussian-log");
+      OBConversion::RegisterFormat("g92",this);
+      OBConversion::RegisterFormat("g94",this);
       OBConversion::RegisterFormat("g98",this);
       OBConversion::RegisterFormat("g03",this);
     }
@@ -34,17 +36,17 @@ namespace OpenBabel
     virtual const char* Description() //required
     {
       return
-        "Gaussian98/03 Output\n \
-       Read Options e.g. -as\n\
-        s  Output single bonds only\n\
-        b  Disable bonding entirely\n\n";
+        "Gaussian98/03 Output\n"
+        "Read Options e.g. -as\n"
+        "  s  Output single bonds only\n"
+        "  b  Disable bonding entirely\n\n";
     };
 
     virtual const char* SpecificationURL()
     { return "http://www.gaussian.com/";};
 
     virtual const char* GetMIMEType() 
-    { return "chemical/x-gaussian"; };
+    { return "chemical/x-gaussian-log"; };
 
     //Flags() can return be any the following combined by | or be omitted if none apply
     // NOTREADABLE  READONEONLY  NOTWRITABLE  WRITEONEONLY
@@ -60,28 +62,37 @@ namespace OpenBabel
   //Make an instance of the format class
   GaussianOutputFormat theGaussianOutputFormat;
 
-  class GaussianInputFormat : public OBFormat
+  class GaussianInputFormat : public OBMoleculeFormat
   {
   public:
     //Register this format type ID
     GaussianInputFormat()
     {
+      OBConversion::RegisterFormat("com",this, "chemical/x-gaussian-input");
       OBConversion::RegisterFormat("gau",this);
-      OBConversion::RegisterFormat("com",this);
-    }
+      OBConversion::RegisterFormat("gjc",this);
+      OBConversion::RegisterFormat("gjf",this);
+      OBConversion::RegisterOptionParam("b", NULL, 0, OBConversion::OUTOPTIONS);
+      // Command-line keywords
+      OBConversion::RegisterOptionParam("k", NULL, 1, OBConversion::OUTOPTIONS);
+      // Command-line keyword file
+      OBConversion::RegisterOptionParam("f", NULL, 1, OBConversion::OUTOPTIONS);    }
 
     virtual const char* Description() //required
     {
       return
-        "Gaussian 98/03 Cartesian Input\n \
-            No comments yet\n";
+        "Gaussian 98/03 Input\n"
+        "Write Options e.g. -xk\n"
+        "  b               Output includes bonds\n"
+        "  k  \"keywords\" Use the specified keywords for input\n"
+        "  f    <file>     Read the file specified for input keywords\n\n";
     };
 
     virtual const char* SpecificationURL()
     {return "http://www.gaussian.com/g_ur/m_input.htm";};
 
     virtual const char* GetMIMEType() 
-    { return "chemical/x-gaussian"; };
+    { return "chemical/x-gaussian-input"; };
 
     //Flags() can return be any the following combined by | or be omitted if none apply
     // NOTREADABLE  READONEONLY  NOTWRITABLE  WRITEONEONLY
@@ -90,31 +101,15 @@ namespace OpenBabel
       return NOTREADABLE | WRITEONEONLY;
     };
 
-    //*** This section identical for most OBMol conversions ***
     ////////////////////////////////////////////////////
     /// The "API" interface functions
     virtual bool WriteMolecule(OBBase* pOb, OBConversion* pConv);
 
-    ////////////////////////////////////////////////////
-    /// The "Convert" interface functions
-    virtual bool WriteChemObject(OBConversion* pConv)
-    {
-      //Retrieve the target OBMol
-      OBBase* pOb = pConv->GetChemObject();
-      OBMol* pmol = dynamic_cast<OBMol*> (pOb);
-      bool ret=false;
-      if(pmol)
-        ret=WriteMolecule(pmol,pConv);
-      delete pOb;
-      return ret;
-    };
   };
-  //***
 
   //Make an instance of the format class
   GaussianInputFormat theGaussianInputFormat;
 
-
   ////////////////////////////////////////////////////////////////
 
   bool GaussianInputFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
@@ -127,20 +122,74 @@ namespace OpenBabel
     ostream &ofs = *pConv->GetOutStream();
     OBMol &mol = *pmol;
 
-    unsigned int i;
     char buffer[BUFF_SIZE];
+    const char *keywords = pConv->IsOption("k",OBConversion::OUTOPTIONS);
+    const char *keywordsEnable = pConv->IsOption("k",OBConversion::GENOPTIONS);
+    const char *keywordFile = pConv->IsOption("f",OBConversion::OUTOPTIONS);
+    string defaultKeywords = "#Put Keywords Here, check Charge and Multiplicity.";
 
-    ofs << "%" << endl << '\045';
-    ofs << "#Put Keywords Here, check Charge and Multiplicity" << endl << endl;
+    if(keywords)
+      {
+        defaultKeywords = keywords;
+      }
+    
+    if (keywordsEnable)
+      {
+        string model;
+        string basis;
+        string method;
+
+        OBPairData *pd = (OBPairData *) pmol->GetData("model");
+        if(pd)
+          model = pd->GetValue();
+
+        pd = (OBPairData *) pmol->GetData("basis");
+        if(pd)
+          basis = pd->GetValue();
+
+        pd = (OBPairData *) pmol->GetData("method");
+        if(pd)
+          method = pd->GetValue();
+
+        if(method == "optimize")
+          {
+            method = "opt";
+          }
+
+        if(model != "" && basis != "" && method != "")
+          {
+            ofs << model << "/" << basis << "," << method << endl;
+          }
+        else
+          {
+            ofs << "#Unable to translate keywords!" << endl;
+            ofs << defaultKeywords << endl;
+          }
+      }
+    else if (keywordFile)
+      {
+        ifstream kfstream(keywordFile);
+        string keyBuffer;
+        if (kfstream)
+          {
+            while (getline(kfstream, keyBuffer))
+              ofs << keyBuffer << endl;
+          }
+      }
+    else 
+      {
+        ofs << defaultKeywords << endl;
+      }
+    ofs << endl; // blank line after keywords
     ofs << " " << mol.GetTitle() << endl << endl;
 
-    OBAtom *atom;
-    string str,str1;
-    snprintf(buffer, BUFF_SIZE, "%d  %d", mol.GetTotalCharge(), mol.GetTotalSpinMultiplicity());
+    snprintf(buffer, BUFF_SIZE, "%d  %d", 
+             mol.GetTotalCharge(),
+             mol.GetTotalSpinMultiplicity());
     ofs << buffer << endl;
-    for(i = 1;i <= mol.NumAtoms(); i++)
+
+    FOR_ATOMS_OF_MOL(atom, mol)
       {
-        atom = mol.GetAtom(i);
         if (atom->GetIsotope() == 0)
           snprintf(buffer, BUFF_SIZE, "%-3s      %10.5f      %10.5f      %10.5f ",
                    etab.GetSymbol(atom->GetAtomicNum()),
@@ -153,6 +202,41 @@ namespace OpenBabel
 	
         ofs << buffer << endl;
       }
+
+    // Bonds, contributed by Daniel Mansfield
+    if (pConv->IsOption("b",OBConversion::OUTOPTIONS))
+    {
+      // first, make begin.GetIdx < end.GetIdx
+      OBBond* bond;
+      OBAtom *atom;
+      vector<OBEdgeBase*>::iterator j;
+      vector<OBNodeBase*>::iterator i;
+      OBAtom *bgn, *end;
+      for (bond = mol.BeginBond(j); bond; bond = mol.NextBond(j)) 
+        {
+          if (bond->GetBeginAtomIdx() > bond->GetEndAtomIdx()) {
+            bgn = bond->GetBeginAtom();
+            end = bond->GetEndAtom();
+            bond->SetBegin(end);
+            bond->SetEnd(bgn);
+          }
+        }
+
+      // this seems inefficient -- perhaps using atom neighbor iterators?
+      // -GRH
+      for (atom = mol.BeginAtom(i);atom;atom = mol.NextAtom(i))
+        {
+          ofs << endl << atom->GetIdx() << " ";
+          for (bond = mol.BeginBond(j); bond; bond = mol.NextBond(j)) 
+            {
+              if (bond->GetBeginAtomIdx() == atom->GetIdx()) {
+                snprintf(buffer, BUFF_SIZE, "%d %1.1f ", bond->GetEndAtomIdx(), (float) bond->GetBondOrder());
+                ofs << buffer;
+              }
+            }
+        } // iterate through atoms
+    } // end writing bonds
+
     // file should end with a blank line
     ofs << endl;
     return(true);
@@ -163,7 +247,7 @@ namespace OpenBabel
   // the openbabel-discuss@lists.sourceforge.net mailing list and/or post a bug
   bool GaussianOutputFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
   {
-    OBMol* pmol = dynamic_cast<OBMol*>(pOb);
+    OBMol* pmol = pOb->CastAndClear<OBMol>();
     if(pmol==NULL)
       return false;
 
@@ -254,8 +338,16 @@ namespace OpenBabel
       mol.PerceiveBondOrders();
 
     mol.EndModify();
-    if (hasPartialCharges)
+    if (hasPartialCharges) {
       mol.SetPartialChargesPerceived();
+
+      // Annotate that partial charges come from Mulliken
+      OBPairData *dp = new OBPairData;
+      dp->SetAttribute("PartialCharges");
+      dp->SetValue("Mulliken");
+      dp->SetOrigin(perceived);
+      mol.SetData(dp);
+    }
     mol.SetTotalCharge(charge);
     mol.SetTotalSpinMultiplicity(spin);
     
