@@ -1,7 +1,7 @@
 /**********************************************************************
 Copyright (C) 1998-2001 by OpenEye Scientific Software, Inc.
 Some portions Copyright (C) 2001-2006 by Geoffrey Hutchison
-Portions Copyright (C) 2004-2005 by Chris Morley
+Portions Copyright (C) 2004-2006 by Chris Morley
 
 This file is part of the Open Babel project.
 For more information, see <http://openbabel.sourceforge.net/>
@@ -15,7 +15,7 @@ but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 ***********************************************************************/
-#include "babelconfig.h"
+#include <openbabel/babelconfig.h>
 
 #ifdef _WIN32
 #pragma warning (disable : 4786)
@@ -25,40 +25,31 @@ GNU General Public License for more details.
 #include <vector>
 #include <iomanip>
 #include <map>
-#include "mol.h"
-#include "obconversion.h"
-#include "obmolecformat.h"
-#include "chiral.h"
+#include <openbabel/obmolecformat.h>
+#include <openbabel/chiral.h>
 
 using namespace std;
 namespace OpenBabel
 {
 
-  class MOLFormat : public OBMoleculeFormat
+  //MDLFormat is a base class which is never instantiated.
+  //MOLFormat and SDFormat are derived from it and have their own constructors.
+  //SDFormat has its own WriteMolecule which sets the "sd" option.
+
+  class MDLFormat : public OBMoleculeFormat
   {
     map<OBAtom*,OBChiralData*> _mapcd; // map of ChiralAtoms and their data
   public:
-    //Register this format type ID
-    MOLFormat() 
-    {
-      OBConversion::RegisterFormat("mol",this, "chemical/x-mdl-molfile");
-      OBConversion::RegisterFormat("mdl",this, "chemical/x-mdl-molfile");
-      OBConversion::RegisterFormat("sd",this, "chemical/x-mdl-sdfile");
-      OBConversion::RegisterFormat("sdf",this, "chemical/x-mdl-sdfile");
-      OBConversion::RegisterOptionParam("2", this);
-      OBConversion::RegisterOptionParam("3", this);
-    }
-
     virtual const char* Description()
     { return
         "MDL MOL format\n \
 Reads and writes V2000 and V3000 versions\n \
 Write Options, e.g. -x3\n \
- 2  output V2000 (default) or\n \
- 3  output V3000 (used for >999 atoms/bonds) \n \
+ 3  output V3000 not V2000 (used for >999 atoms/bonds) \n \
  m  write no properties\n \
 ";
     };
+/* 2  output V2000 (default) or\n \ */
 
     virtual const char* SpecificationURL()
     {return "http://www.mdl.com/downloads/public/ctfile/ctfile.jsp";};
@@ -97,19 +88,54 @@ Write Options, e.g. -x3\n \
     bool ReadBondBlock(istream& ifs,OBMol& mol, OBConversion* pConv);
     bool WriteV3000(ostream& ofs,OBMol& mol, OBConversion* pConv);
   private:
-    bool HasProperties;
+    bool  HasProperties;
     char* GetTimeDate(char* td);
+    bool  ParseAliasText(OBMol& mol, char* txt, int atomnumber);
+
     map<int,int> indexmap; //relates index in file to index in OBMol
     vector<string> vs;
   };
 
+//**************************************
+class MOLFormat : public MDLFormat
+{
+public:
+  //Register this format type ID
+  MOLFormat() 
+  {
+    OBConversion::RegisterFormat("mol",this, "chemical/x-mdl-molfile");
+    OBConversion::RegisterFormat("mdl",this, "chemical/x-mdl-molfile");
+    OBConversion::RegisterOptionParam("2", this);
+    OBConversion::RegisterOptionParam("3", this);
+  }
+};
   //Make an instance of the format class
   MOLFormat theMOLFormat;
 
-  /////////////////////////////////////////////////////////////////
-  bool MOLFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
+//*************************************
+class SDFormat : public MDLFormat
+{
+public:
+  SDFormat()
   {
-    OBMol* pmol = dynamic_cast<OBMol*>(pOb);
+     OBConversion::RegisterFormat("sd",this, "chemical/x-mdl-sdfile");
+     OBConversion::RegisterFormat("sdf",this, "chemical/x-mdl-sdfile");
+  }
+
+  virtual bool WriteMolecule(OBBase* pOb, OBConversion* pConv)
+  {
+    //The sd option ensures that a $$$$ is written at the end of the file
+    pConv->AddOption("sd",OBConversion::OUTOPTIONS);
+    return MDLFormat::WriteMolecule(pOb, pConv);
+  }
+};
+//Make an instance of the format class
+  SDFormat theSDFormat;
+
+  /////////////////////////////////////////////////////////////////
+  bool MDLFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
+  {
+    OBMol* pmol = pOb->CastAndClear<OBMol>();
 
     //Define some references so we can use the old parameter names
     istream &ifs = *pConv->GetInStream();
@@ -272,15 +298,29 @@ Write Options, e.g. -x3\n \
               return true;
             if(!strncmp(buffer,"M  END",6))
               break;
-            if(strncmp(buffer,"M  CHG",6) && strncmp(buffer,"M  RAD",6) && strncmp(buffer,"M  ISO",6))
-              continue;
             if(!strncmp(buffer,"S  SKP",6))
               {
                 int i = atoi(buffer+6);
                 for(;i>0;--i)
                   ifs.getline(buffer,BUFF_SIZE);
-                break;
+                continue;
               }
+            if(buffer[0]=='A') //alias
+            {
+              //Parses entries like 
+              //A  6
+              //CD2
+              int atomnum = atoi(buffer+2);
+              ifs.getline(buffer,BUFF_SIZE);
+              if(!ParseAliasText(mol, buffer, atomnum))
+              {
+                obErrorLog.ThrowError(__FUNCTION__, "Error in alias block", obError);
+                return false;
+              }
+              continue;
+            }
+            if(strncmp(buffer,"M  CHG",6) && strncmp(buffer,"M  RAD",6) && strncmp(buffer,"M  ISO",6))
+              continue;
             r1 = buffer;
             int n = atoi((r1.substr(6,3)).c_str()); //entries on this line
             if(n==0) break;
@@ -300,8 +340,8 @@ Write Options, e.g. -x3\n \
                   at->SetIsotope(value);
                 //Although not done here,according to the specification, 
                 //previously set formal charges should be reset to zero
+                }
                 // Lines setting several other properties are not implemented
-              }
           }
       }
     mol.AssignSpinMultiplicity();
@@ -320,13 +360,14 @@ Write Options, e.g. -x3\n \
             atom=ChiralSearch->first;
             cd=ChiralSearch->second;
             atom->SetData(cd);
-          }
+          }    
       }
 
     if (comment.length())
       {
         OBCommentData *cd = new OBCommentData;
         cd->SetData(comment);
+        cd->SetOrigin(fileformatInput);
         mol.SetData(cd);
       }
         
@@ -339,7 +380,7 @@ Write Options, e.g. -x3\n \
             size_t lt=buff.find("<")+1;
             size_t rt = buff.find_last_of(">");
             string attr = buff.substr(lt,rt-lt);
- 
+
             // sometimes we can hit more data than BUFF_SIZE, so we'll use a std::string
             string line;
             buff.clear();
@@ -355,27 +396,31 @@ Write Options, e.g. -x3\n \
                   break;
               }
             Trim(buff);
- 
+
             OBPairData *dp = new OBPairData;
             dp->SetAttribute(attr);
             dp->SetValue(buff);
+            dp->SetOrigin(fileformatInput);
             mol.SetData(dp);
+            
+            if(!strcasecmp(attr.c_str(),"NAME") && *mol.GetTitle()=='\0')
+              mol.SetTitle(buff);
           }
         if (!strncmp(buffer,"$$$$",4)) break;
         if (!strncmp(buffer,"$MOL",4)) break;
       }
-  
+
     if (!setDimension && mol.Has3D())
       mol.SetDimension(3);
     else if (!setDimension && !mol.Has3D())
       mol.SetDimension(2);
-  
+
     return(true);
 
   }
 
   /////////////////////////////////////////////////////////////////
-  bool MOLFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
+  bool MDLFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
   {
     OBMol* pmol = dynamic_cast<OBMol*>(pOb);
 
@@ -417,11 +462,7 @@ Write Options, e.g. -x3\n \
 
         if (mol.NumAtoms() > 999 || mol.NumBonds() > 999) // Three digits!
           {
-#ifdef HAVE_SSTREAM
             stringstream errorMsg;
-#else
-            strstream errorMsg;
-#endif
             errorMsg << "MDL Molfile conversion failed: Molecule is too large to convert." << endl;
             errorMsg << "  File format (v2000) is limited to 999 atoms or bonds." << endl;
             errorMsg << "  Molecule size: " << mol.NumAtoms() << " atoms ";
@@ -447,7 +488,7 @@ Write Options, e.g. -x3\n \
         ofs << buff << endl;
 
         OBAtom *atom;
-        vector<OBNodeBase*>::iterator i;
+        vector<OBAtom*>::iterator i;
         int charge;
         for (atom = mol.BeginAtom(i);atom;atom = mol.NextAtom(i)) {
           switch (atom->GetFormalCharge()) {
@@ -473,7 +514,7 @@ Write Options, e.g. -x3\n \
         //so the bonds come out sorted
         OBAtom *nbr;
         OBBond *bond;
-        vector<OBEdgeBase*>::iterator j;
+        vector<OBBond*>::iterator j;
         for (atom = mol.BeginAtom(i);atom;atom = mol.NextAtom(i))
           for (nbr = atom->BeginNbrAtom(j);nbr;nbr = atom->NextNbrAtom(j))
             if (atom->GetIdx() < nbr->GetIdx()) {
@@ -521,7 +562,8 @@ Write Options, e.g. -x3\n \
 
     ofs << "M  END" << endl;
 
-    if(!pConv->IsOption("m")) //No properties output if option m
+    //For SD files only, write properties unless option m
+    if(pConv->IsOption("sd") && !pConv->IsOption("m"))
       {
         vector<OBGenericData*>::iterator k;
         vector<OBGenericData*> vdata = mol.GetData();
@@ -537,9 +579,10 @@ Write Options, e.g. -x3\n \
       }
         
     //Unless option no$$$$ is set, $$$$ is always written between molecules and
-    //at the end any if properties have been output in any molecule.
+    //at the end any if properties have been output in any molecule,
+    //or if the sd option is set.
     if(!pConv->IsOption("no$$$$"))
-      if(!pConv->IsLast()  || HasProperties  )  
+      if(!pConv->IsLast()  || HasProperties  || pConv->IsOption("sd"))  
         ofs << "$$$$" << endl;
 
     return(true);
@@ -547,7 +590,7 @@ Write Options, e.g. -x3\n \
 
 
   //////////////////////////////////////////////////////
-  bool MOLFormat::ReadV3000Block(istream& ifs, OBMol& mol, OBConversion* pConv,bool DoMany)
+  bool MDLFormat::ReadV3000Block(istream& ifs, OBMol& mol, OBConversion* pConv,bool DoMany)
   {
     do
       {
@@ -589,7 +632,7 @@ Write Options, e.g. -x3\n \
   }
 
   //////////////////////////////////////////////////////
-  bool MOLFormat::ReadV3000Line(istream& ifs, vector<string>& vs)
+  bool MDLFormat::ReadV3000Line(istream& ifs, vector<string>& vs)
   {
     char buffer[BUFF_SIZE];
     if(!ifs.getline(buffer,BUFF_SIZE)) return false;
@@ -607,7 +650,7 @@ Write Options, e.g. -x3\n \
   }
 
   //////////////////////////////////////////////////////
-  bool MOLFormat::ReadAtomBlock(istream& ifs,OBMol& mol, OBConversion* pConv)
+  bool MDLFormat::ReadAtomBlock(istream& ifs,OBMol& mol, OBConversion* pConv)
   {     
     OBAtom atom;
     bool chiralWatch=false;
@@ -675,7 +718,7 @@ Write Options, e.g. -x3\n \
   }
 
   //////////////////////////////////////////////////////
-  bool MOLFormat::ReadBondBlock(istream& ifs,OBMol& mol, OBConversion* pConv)
+  bool MDLFormat::ReadBondBlock(istream& ifs,OBMol& mol, OBConversion* pConv)
   {
     for(;;)
       {
@@ -732,7 +775,7 @@ Write Options, e.g. -x3\n \
   }
 
   //////////////////////////////////////////////////////////
-  bool MOLFormat::WriteV3000(ostream& ofs,OBMol& mol, OBConversion* pConv)
+  bool MDLFormat::WriteV3000(ostream& ofs,OBMol& mol, OBConversion* pConv)
   {
     // Check to see if there are any untyped aromatic bonds (GetBO == 5)
     // These must be kekulized first
@@ -754,7 +797,7 @@ Write Options, e.g. -x3\n \
     ofs << "M  V30 BEGIN ATOM" <<endl;
     OBAtom *atom;
     int index=1;
-    vector<OBNodeBase*>::iterator i;
+    vector<OBAtom*>::iterator i;
     for (atom = mol.BeginAtom(i);atom;atom = mol.NextAtom(i))
       {  
         ofs     << "M  V30 "
@@ -782,7 +825,7 @@ Write Options, e.g. -x3\n \
                 OBAtom *nbr;
                 int Hid = (mol.NumAtoms()+1) ;// max Atom ID +1 
                 vector<unsigned int> nbr_atms;
-                vector<OBEdgeBase*>::iterator i;
+                vector<OBBond*>::iterator i;
                 for (nbr = atom->BeginNbrAtom(i);nbr;nbr = atom->NextNbrAtom(i))
                   {
                     if (nbr->IsHydrogen()){Hid=nbr->GetIdx();continue;}
@@ -828,7 +871,7 @@ Write Options, e.g. -x3\n \
     index=1;
     OBAtom *nbr;
     OBBond *bond;
-    vector<OBEdgeBase*>::iterator j;
+    vector<OBBond*>::iterator j;
     for (atom = mol.BeginAtom(i);atom;atom = mol.NextAtom(i))
       {
         for (nbr = atom->BeginNbrAtom(j);nbr;nbr = atom->NextNbrAtom(j))
@@ -855,17 +898,81 @@ Write Options, e.g. -x3\n \
     return true;
   }
 
-  char* MOLFormat::GetTimeDate(char* td)
+  char* MDLFormat::GetTimeDate(char* td)
   {
     //returns MMDDYYHHmm
     struct tm* ts;
     time_t long_time;
     time( &long_time );
     ts = localtime( &long_time ); 
-    snprintf(td, sizeof(td), "%02d%02d%02d%02d%02d", ts->tm_mon+1, ts->tm_mday, 
+    snprintf(td, 11, "%02d%02d%02d%02d%02d", ts->tm_mon+1, ts->tm_mday, 
              ((ts->tm_year>=100)? ts->tm_year-100 : ts->tm_year),
              ts->tm_hour, ts->tm_min);
     return td;
   }
 
+  bool MDLFormat::ParseAliasText(OBMol& mol, char* txt, int atomnumber)
+  {
+    //Crude implementation
+    //Only single character element symbols are handled
+    //Atom which replaces atomnumber is the first non-H 
+    //Will parse ND2 DS CH-
+    if(!isalpha(*txt)) //first char is the element that replaces atomnumber
+      return false;
+    //Swaps any leading H isotope with the first non-H atom
+    if(*txt=='H' || *txt=='D' || *txt=='T')
+    {
+      char* p =txt+1;
+      while(*p && *p=='H' && *p=='D' && *p=='T')p++;
+      if(*p)
+        std::swap(*p, *txt);
+    }
+    char symb[2];
+    symb[0]=*(txt++);
+    symb[1]='\0';
+    OBAtom* pAtom = mol.GetAtom(atomnumber);
+    if(!pAtom)
+      return false;
+    int iso = 0;
+    pAtom->SetAtomicNum(etab.GetAtomicNum(symb,iso));
+    if(iso)
+      pAtom->SetIsotope(iso);
+
+    while(*txt)
+    {
+      if(isspace(*txt)) {
+        ++txt;
+        continue;
+      }
+      int chg=0;
+      if(*txt=='-')
+        chg = -1;
+      else if(*txt=='+')
+        chg = 1;
+      if(chg)
+      {
+        pAtom->SetFormalCharge(pAtom->GetFormalCharge()+chg);//put on central atom e.g. CH-
+        ++txt;
+        continue;
+      }
+      if(!isalpha(*txt))
+        return false;
+      symb[0]=*txt;
+      int rep = atoi(++txt);
+      if(rep)
+        ++txt;
+      do //for each rep
+      {
+        OBAtom* newAtom = mol.NewAtom();
+        iso = 0;
+        newAtom->SetAtomicNum(etab.GetAtomicNum(symb,iso));
+        if(iso)
+          newAtom->SetIsotope(iso);
+
+        if (!mol.AddBond(atomnumber,mol.NumAtoms(),1,0)) return false;
+      }while(--rep>0);
+    }
+    return true;
+  }
+
 }
