@@ -11,10 +11,9 @@ but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 ***********************************************************************/
+#include <openbabel/babelconfig.h>
 
-#include "mol.h"
-#include "obconversion.h"
-#include "obmolecformat.h"
+#include <openbabel/obmolecformat.h>
 
 using namespace std;
 namespace OpenBabel
@@ -22,101 +21,86 @@ namespace OpenBabel
 
 #define BOHR_TO_ANGSTROM 0.529177249
 
-class MPQCFormat : public OBMoleculeFormat
-{
-public:
+  class MPQCFormat : public OBMoleculeFormat
+  {
+  public:
     //Register this format type ID
     MPQCFormat()
     {
-        OBConversion::RegisterFormat("mpqc",this);
+      OBConversion::RegisterFormat("mpqc",this);
     }
 
-  virtual const char* Description() //required
-  {
-    return
-      "MPQC output format\n \
+    virtual const char* Description() //required
+    {
+      return
+        "MPQC output format\n \
        Read Options e.g. -as\n\
         s  Output single bonds only\n\
         b  Disable bonding entirely\n\n";
-  };
+    };
 
-  virtual const char* SpecificationURL()
-  { return "http://www.mpqc.org/mpqc-html/mpqcinp.html";}; //optional
+    virtual const char* SpecificationURL()
+    { return "http://www.mpqc.org/mpqc-html/mpqcinp.html";}; //optional
 
     //Flags() can return be any the following combined by | or be omitted if none apply
     // NOTREADABLE  READONEONLY  NOTWRITABLE  WRITEONEONLY
     virtual unsigned int Flags()
     {
-        return NOTWRITABLE | READONEONLY;
+      return NOTWRITABLE | READONEONLY;
     };
 
     ////////////////////////////////////////////////////
     /// The "API" interface functions
     virtual bool ReadMolecule(OBBase* pOb, OBConversion* pConv);
-};
+  };
 
-//Make an instance of the format class
-MPQCFormat theMPQCFormat;
+  //Make an instance of the format class
+  MPQCFormat theMPQCFormat;
 
 
 
-class MPQCInputFormat : public OBFormat
-{
-public:
+  class MPQCInputFormat : public OBMoleculeFormat
+  {
+  public:
     //Register this format type ID
     MPQCInputFormat()
     {
-        OBConversion::RegisterFormat("mpqcin",this);
+      OBConversion::RegisterFormat("mpqcin",this);
     }
 
     virtual const char* Description() //required
     {
-        return
-            "MPQC simplified input format\n \
+      return
+        "MPQC simplified input format\n \
             No comments yet\n";
     };
 
-  virtual const char* SpecificationURL()
-  { return "http://www.mpqc.org/mpqc-html/mpqcinp.html";}; //optional
+    virtual const char* SpecificationURL()
+    { return "http://www.mpqc.org/mpqc-html/mpqcinp.html";}; //optional
 
     //Flags() can return be any the following combined by | or be omitted if none apply
     // NOTREADABLE  READONEONLY  NOTWRITABLE  WRITEONEONLY
     virtual unsigned int Flags()
     {
-        return NOTREADABLE | WRITEONEONLY;
+      return NOTREADABLE | WRITEONEONLY;
     };
 
-    //*** This section identical for most OBMol conversions ***
     ////////////////////////////////////////////////////
     /// The "API" interface functions
     virtual bool WriteMolecule(OBBase* pOb, OBConversion* pConv);
 
-    ////////////////////////////////////////////////////
-    /// The "Convert" interface functions
-    virtual bool WriteChemObject(OBConversion* pConv)
-    {
-        //Retrieve the target OBMol
-        OBBase* pOb = pConv->GetChemObject();
-        OBMol* pmol = dynamic_cast<OBMol*> (pOb);
-        bool ret=false;
-        if(pmol)
-            ret=WriteMolecule(pmol,pConv);
-        delete pOb;
-        return ret;
-    };
-};
-//***
+  };
 
-//Make an instance of the format class
-MPQCInputFormat theMPQCInputFormat;
+  //Make an instance of the format class
+  MPQCInputFormat theMPQCInputFormat;
 
-/////////////////////////////////////////////////////////////////
-bool MPQCFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
-{
+  /////////////////////////////////////////////////////////////////
+  bool MPQCFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
+  {
 
-    OBMol* pmol = dynamic_cast<OBMol*>(pOb);
+    OBMol* pmol = pOb->CastAndClear<OBMol>();
     if(pmol==NULL)
-        return false;
+      return false;
 
     //Define some references so we can use the old parameter names
     istream &ifs = *pConv->GetInStream();
@@ -132,44 +116,44 @@ bool MPQCFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
 
     mol.BeginModify();
     while	(ifs.getline(buffer,BUFF_SIZE))
-    {
+      {
         if(strstr(buffer,"<Molecule>:") != NULL)
-        {
+          {
             // mol.EndModify();
             mol.Clear();
             while	(strstr(buffer,"geometry") == NULL)
-            {
+              {
                 if (strstr(buffer,"angstrom") != NULL)
-                    bohr = false;
+                  bohr = false;
                 if (!ifs.getline(buffer,BUFF_SIZE))
-                    return(false);
-            }
+                  return(false);
+              }
             ifs.getline(buffer,BUFF_SIZE); // Now we're on the atoms
             tokenize(vs,buffer);
             while (vs.size() == 6)
-            {
+              {
                 if (bohr)
-                {
+                  {
                     x = atof((char*)vs[3].c_str()) * BOHR_TO_ANGSTROM;
                     y = atof((char*)vs[4].c_str()) * BOHR_TO_ANGSTROM;
                     z = atof((char*)vs[5].c_str()) * BOHR_TO_ANGSTROM;
-                }
+                  }
                 else
-                {
+                  {
                     x = atof((char*)vs[3].c_str());
                     y = atof((char*)vs[4].c_str());
                     z = atof((char*)vs[5].c_str());
-                }
+                  }
                 atom = mol.NewAtom();
                 atom->SetVector(x,y,z);
                 atom->SetAtomicNum(etab.GetAtomicNum(vs[1].c_str()));
 
                 if (!ifs.getline(buffer,BUFF_SIZE))
-                    break;
+                  break;
                 tokenize(vs,buffer);
-            }
-        }
-    }
+              }
+          }
+      }
 
     if (!pConv->IsOption("b",OBConversion::INOPTIONS))
       mol.ConnectTheDots();
@@ -181,19 +165,19 @@ bool MPQCFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
     mol.SetTitle(title);
 
     return(true);
-}
+  }
 
-bool MPQCInputFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
-{
+  bool MPQCInputFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
+  {
     OBMol* pmol = dynamic_cast<OBMol*>(pOb);
     if(pmol==NULL)
-        return false;
+      return false;
 
     //Define some references so we can use the old parameter names
     ostream &ofs = *pConv->GetOutStream();
     OBMol &mol = *pmol;
 
-    unsigned int i;
+    //   unsigned int i;
     char buffer[BUFF_SIZE];
 
     ofs << "% " << mol.GetTitle() << "\n";
@@ -201,17 +185,17 @@ bool MPQCInputFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
     ofs << "molecule:\n";
 
     FOR_ATOMS_OF_MOL(atom, mol)
-    {
+      {
         snprintf(buffer, BUFF_SIZE, "%4s  %8.5f  %8.5f  %8.5f \n",
                 etab.GetSymbol(atom->GetAtomicNum()),
                 atom->GetX(),
                 atom->GetY(),
                 atom->GetZ());
         ofs << buffer;
-    }
+      }
 
     ofs << "\n\n\n";
     return(true);
-}
+  }
 
 } //namespace OpenBabel
