@@ -1,6 +1,6 @@
 /**********************************************************************
 Copyright (C) 1998-2001 by OpenEye Scientific Software, Inc.
-Some portions Copyright (C) 2003-2005 Geoffrey R. Hutchison
+Some portions Copyright (C) 2003-2006 Geoffrey R. Hutchison
 Some portions Copyright (C) 2004 by Chris Morley
  
 This program is free software; you can redistribute it and/or modify
@@ -13,10 +13,8 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 ***********************************************************************/
 
-#include "babelconfig.h"
-#include "mol.h"
-#include "obconversion.h"
-#include "obmolecformat.h"
+#include <openbabel/babelconfig.h>
+#include <openbabel/obmolecformat.h>
 
 #include <vector>
 #include <map>
@@ -40,10 +38,10 @@ namespace OpenBabel
     virtual const char* Description() //required
     {
       return
-        "Protein Data Bank format\n \
-       Read Options e.g. -as\n\
-        s  Output single bonds only\n\
-        b  Disable bonding entirely\n\n";
+        "Protein Data Bank format\n"
+        "Read Options e.g. -as\n"
+        "  s  Output single bonds only\n"
+        "  b  Disable bonding entirely\n\n";
     };
 
     virtual const char* SpecificationURL()
@@ -83,7 +81,7 @@ namespace OpenBabel
   bool PDBFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
   {
 
-    OBMol* pmol = dynamic_cast<OBMol*>(pOb);
+    OBMol* pmol = pOb->CastAndClear<OBMol>();
     if(pmol==NULL)
       return false;
 
@@ -97,6 +95,7 @@ namespace OpenBabel
     OBBitVec bs;
 
     mol.SetTitle(title);
+    mol.SetChainsPerceived(); // It's a PDB file, we read all chain/res info.
 
     mol.BeginModify();
     while (ifs.getline(buffer,BUFF_SIZE) && !EQn(buffer,"END",3))
@@ -114,33 +113,33 @@ namespace OpenBabel
           ParseConectRecord(buffer,mol);
       }
 
+    if (!mol.NumAtoms()) { // skip the rest of this processing
+      mol.EndModify();
+      return(false);
+    }
+
     resdat.AssignBonds(mol,bs);
     /*assign hetatm bonds based on distance*/
 
+    mol.EndModify();
+
     if (!pConv->IsOption("b",OBConversion::INOPTIONS))
       mol.ConnectTheDots();
 
-      if (!pConv->IsOption("s",OBConversion::INOPTIONS) && !pConv->IsOption("b",OBConversion::INOPTIONS))
-        mol.PerceiveBondOrders();
+    if (!pConv->IsOption("s",OBConversion::INOPTIONS) && !pConv->IsOption("b",OBConversion::INOPTIONS))
+      mol.PerceiveBondOrders();
 
     // clean out remaining blank lines
     while(ifs.peek() != EOF && ifs.good() && 
           (ifs.peek() == '\n' || ifs.peek() == '\r'))
       ifs.getline(buffer,BUFF_SIZE);
 
-    mol.EndModify();
-
-    mol.SetAtomTypesPerceived();
-    atomtyper.AssignImplicitValence(mol);
-
-    if (!mol.NumAtoms())
-      return(false);
     return(true);
   }
 
   ////////////////////////////////////////////////////////////////
   static bool ParseAtomRecord(char *buffer, OBMol &mol,int chainNum)
-    /* ATOMFORMAT "(i5,1x,a4,a1,a3,1x,a1,i4,a1,3x,3f8.3,2f6.2,1x,i3)" */
+  /* ATOMFORMAT "(i5,1x,a4,a1,a3,1x,a1,i4,a1,3x,3f8.3,2f6.2,1x,i3)" */
   {
     string sbuf = &buffer[6];
     if (sbuf.size() < 48)
@@ -183,25 +182,17 @@ namespace OpenBabel
           resname = resname.substr(0,resname.size()-1);
       }
 
-    /* residue sequence number */
-
-    string resnum = sbuf.substr(16,4);
-
-    /* X, Y, Z */
-    string xstr = sbuf.substr(24,8);
-    string ystr = sbuf.substr(32,8);
-    string zstr = sbuf.substr(40,8);
-
     string type;
-
     if (EQn(buffer,"ATOM",4))
       {
         type = atmid.substr(0,2);
-        if (isdigit(type[0]))
-          type = atmid.substr(1,1);
-        else if (sbuf[6] == ' ' &&
-                 strncasecmp(type.c_str(), "Zn", 2) != 0 &&
-                 strncasecmp(type.c_str(), "Fe", 2) != 0)
+        if (isdigit(type[0])) {
+          // sometimes non-standard files have, e.g 11HH
+          if (!isdigit(type[1])) type = atmid.substr(1,1);
+          else type = atmid.substr(2,1); 
+        } else if (sbuf[6] == ' ' &&
+                   strncasecmp(type.c_str(), "Zn", 2) != 0 &&
+                   strncasecmp(type.c_str(), "Fe", 2) != 0)
           type = atmid.substr(0,1);     // one-character element
         
 
@@ -240,13 +231,19 @@ namespace OpenBabel
           }
         else
           {
-            if (isalpha(atmid[0]))
-              type = atmid.substr(0,2);
+            if (isalpha(atmid[0])) {
+              
+              if (atmid[2] == '\0' || atmid[2] == ' ') type = atmid.substr(0,2);
+              else if (atmid[0] == 'A') // alpha prefix
+                type = atmid.substr(1, atmid.size() - 1);
+              else type = atmid.substr(0,1);
+            }
             else if (atmid[0] == ' ')
               type = atmid.substr(1,1); // one char element
             else
               type = atmid.substr(1,2);
 
+            // Some cleanup steps
             if (atmid == resname)
               {
                 type = atmid;
@@ -256,7 +253,7 @@ namespace OpenBabel
             else
               if (resname == "ADR" || resname == "COA" || resname == "FAD" ||
                   resname == "GPG" || resname == "NAD" || resname == "NAL" ||
-                  resname == "NDP")
+                  resname == "NDP" || resname == "ABA")
                 {
                   if (type.size() > 1)
                     type = type.substr(0,1);
@@ -266,27 +263,37 @@ namespace OpenBabel
                 if (isdigit(type[0]))
                   {
                     type = type.substr(1,1);
-                    //type.erase(0,1);
-                    //if (type.size() > 1) type.erase(1,type.size()-1);
                   }
                 else
                   if (type.size() > 1 && isdigit(type[1]))
                     type = type.substr(0,1);
-            //type.erase(1,1);
                   else
-                    if (type.size() > 1 && isalpha(type[1]) && isupper(type[1]))
-                      type[1] = tolower(type[1]);
+                    if (type.size() > 1 && isalpha(type[1])) {
+                      if (type[0] == 'O' && type[1] == 'H')
+                        type = type.substr(0,1); // no "Oh" element (e.g. 1MBN)
+                      else if(isupper(type[1]))
+                        {
+                          type[1] = tolower(type[1]);
+                        }
+                    }
           }
         
       }
 
     OBAtom atom;
+    /* X, Y, Z */
+    string xstr = sbuf.substr(24,8);
+    string ystr = sbuf.substr(32,8);
+    string zstr = sbuf.substr(40,8);
     vector3 v(atof(xstr.c_str()),atof(ystr.c_str()),atof(zstr.c_str()));
     atom.SetVector(v);
 
+    // useful for debugging unknown atom types (e.g., PR#1577238)
+    //    cout << mol.NumAtoms() + 1 << " " << atmid << " type: " << type << endl;
     atom.SetAtomicNum(etab.GetAtomicNum(type.c_str()));
-    atom.SetType(type);
 
+    /* residue sequence number */
+    string resnum = sbuf.substr(16,4);
     int        rnum = atoi(resnum.c_str());
     OBResidue *res  = (mol.NumResidues() > 0) ? mol.GetResidue(mol.NumResidues()-1) : NULL;
     if (res == NULL || res->GetName() != resname || static_cast<int>(res->GetNum())
@@ -300,16 +307,14 @@ namespace OpenBabel
 
         if (res == NULL)
           {
-            res = mol.NewResidue()
-              ;
+            res = mol.NewResidue();
             res->SetChainNum(chainNum);
             res->SetName(resname);
             res->SetNum(rnum);
           }
       }
 
-    if (!mol.AddAtom(atom)
-        )
+    if (!mol.AddAtom(atom))
       return(false);
     else
       {
@@ -403,7 +408,7 @@ namespace OpenBabel
                  << "  the record should have 70 columns, but OpenBabel found "
                  << strlen(buffer) << " columns." << endl;
         obErrorLog.ThrowError(__FUNCTION__, errorMsg.str() , obInfo);
-	errorMsg.str(clearError);
+        errorMsg.str(clearError);
       }
 
     // Serial number of the first atom, read from column 7-11 of the
@@ -430,75 +435,80 @@ namespace OpenBabel
 
     if (mol.NumAtoms() <= 9999)
       {
-	// make sure we don't look at salt bridges or whatever, so cut the buffer short
-	buffer[32] = '\0';
-	tokenize(vs,buffer);
-	if( vs.empty() || vs.size() < 2) 
-	  return false;
-	vs.erase(vs.begin()); // remove "CONECT"
+        // make sure we don't look at salt bridges or whatever, so cut the buffer short
+        buffer[32] = '\0';
+        tokenize(vs,buffer);
+        if( vs.empty() || vs.size() < 2) 
+          return false;
+        vs.erase(vs.begin()); // remove "CONECT"
 
-	startAtomSerialNumber = atoi(vs[0].c_str());
+        startAtomSerialNumber = atoi(vs[0].c_str());
       }
     else
       {
-	if (readIntegerFromRecord(buffer, 7, &startAtomSerialNumber) == false)
-	  {
-	    errorMsg << "WARNING: Problems reading a PDB file\n"
-		     << "  Problems reading a CONECT record.\n"
-		     << "  According to the PDB specification,\n"
-		     << "  columns 7-11 should contain the serial number of an atom.\n"
-		     << "  THIS CONECT RECORD WILL BE IGNORED." << endl;
-	    obErrorLog.ThrowError(__FUNCTION__, errorMsg.str() , obWarning);
-	    return(false);
-	  }
+        if (readIntegerFromRecord(buffer, 7, &startAtomSerialNumber) == false)
+          {
+            errorMsg << "WARNING: Problems reading a PDB file\n"
+                     << "  Problems reading a CONECT record.\n"
+                     << "  According to the PDB specification,\n"
+                     << "  columns 7-11 should contain the serial number of an atom.\n"
+                     << "  THIS CONECT RECORD WILL BE IGNORED." << endl;
+            obErrorLog.ThrowError(__FUNCTION__, errorMsg.str() , obWarning);
+            return(false);
+          }
       }
 
-    vector<OBNodeBase*>::iterator i;
-    for (OBAtom *a1 = mol.BeginAtom(i);a1;a1 = mol.NextAtom(i))
-      if (static_cast<long int>(a1->GetResidue()->
-				GetSerialNum(a1)) == startAtomSerialNumber)
-	{
-	  firstAtom = a1;
-	  break;
-	}
+    vector<OBAtom*>::iterator i;
+    for (OBAtom *a1 = mol.BeginAtom(i);a1;a1 = mol.NextAtom(i)) {
+      // atoms may not have residue information, but if they do,
+      // check serial numbers
+      if (a1->GetResidue() != NULL && 
+          static_cast<long int>(a1->GetResidue()->
+                                GetSerialNum(a1)) == startAtomSerialNumber)
+        {
+          firstAtom = a1;
+          break;
+        }
+    }
+
     if (firstAtom == NULL)
       {
-	errorMsg << "WARNING: Problems reading a PDB file:\n"
-		 << "  Problems reading a CONECT record.\n"
-		 << "  According to the PDB specification,\n"
-		 << "  columns 7-11 should contain the serial number of an atom.\n"
-		 << "  No atom was found with this serial number.\n"
-		 << "  THIS CONECT RECORD WILL BE IGNORED." << endl;
-	obErrorLog.ThrowError(__FUNCTION__, errorMsg.str() , obWarning);
-	return(false);
+        errorMsg << "WARNING: Problems reading a PDB file:\n"
+                 << "  Problems reading a CONECT record.\n"
+                 << "  According to the PDB specification,\n"
+                 << "  columns 7-11 should contain the serial number of an atom.\n"
+                 << "  No atom was found with this serial number.\n"
+                 << "  THIS CONECT RECORD WILL BE IGNORED." << endl;
+        obErrorLog.ThrowError(__FUNCTION__, errorMsg.str() , obWarning);
+        return(false);
       }
 
     if (mol.NumAtoms() < 9999)
       {
-	if (vs.size() > 1) boundedAtomsSerialNumbers[0] = atoi(vs[1].c_str());
-	if (vs.size() > 2) boundedAtomsSerialNumbers[1] = atoi(vs[2].c_str());
-	if (vs.size() > 3) boundedAtomsSerialNumbers[2] = atoi(vs[3].c_str());
-	if (vs.size() > 4) boundedAtomsSerialNumbers[3] = atoi(vs[4].c_str());
+        if (vs.size() > 1) boundedAtomsSerialNumbers[0] = atoi(vs[1].c_str());
+        if (vs.size() > 2) boundedAtomsSerialNumbers[1] = atoi(vs[2].c_str());
+        if (vs.size() > 3) boundedAtomsSerialNumbers[2] = atoi(vs[3].c_str());
+        if (vs.size() > 4) boundedAtomsSerialNumbers[3] = atoi(vs[4].c_str());
 
-	unsigned int limit = 4;
-	if (vs.size() <= 4)
-	  limit = vs.size() - 1;
+        unsigned int limit = 4;
+        if (vs.size() <= 4)
+          limit = vs.size() - 1;
 
-	for (unsigned int i = 0; i < limit; i++)
-	  boundedAtomsSerialNumbersValid[i] = true;
+        for (unsigned int i = 0; i < limit; i++)
+          boundedAtomsSerialNumbersValid[i] = true;
       }
     else
       {
-	// Now read the serial numbers. If the first serial number is not
-	// present, this connect record probably contains only hydrogen
-	// bonds and salt bridges, which we ignore. In that case, we just
-	// exit gracefully.
-	boundedAtomsSerialNumbersValid[0] = readIntegerFromRecord(buffer, 12, boundedAtomsSerialNumbers+0);
-	if (boundedAtomsSerialNumbersValid[0] == false)
-	  return(true);
-	boundedAtomsSerialNumbersValid[1] = readIntegerFromRecord(buffer, 17, boundedAtomsSerialNumbers+1);
-	boundedAtomsSerialNumbersValid[2] = readIntegerFromRecord(buffer, 22, boundedAtomsSerialNumbers+2);
-	boundedAtomsSerialNumbersValid[3] = readIntegerFromRecord(buffer, 27, boundedAtomsSerialNumbers+3);
+        // Now read the serial numbers. If the first serial number is not
+        // present, this connect record probably contains only hydrogen
+        // bonds and salt bridges, which we ignore. In that case, we just
+        // exit gracefully.
+        boundedAtomsSerialNumbersValid[0] = readIntegerFromRecord(buffer, 12, boundedAtomsSerialNumbers+0);
+        if (boundedAtomsSerialNumbersValid[0] == false)
+          return(true);
+        boundedAtomsSerialNumbersValid[1] = readIntegerFromRecord(buffer, 17, boundedAtomsSerialNumbers+1);
+        boundedAtomsSerialNumbersValid[2] = readIntegerFromRecord(buffer, 22, boundedAtomsSerialNumbers+2);
+        boundedAtomsSerialNumbersValid[3] = readIntegerFromRecord(buffer, 27, boundedAtomsSerialNumbers+3);
       }
 
     // Now iterate over the VALID boundedAtomsSerialNumbers and connect
@@ -507,13 +517,16 @@ namespace OpenBabel
       {
         // Find atom that is connected to, write an error message
         OBAtom *connectedAtom = 0L;
-        for (OBAtom *a1 = mol.BeginAtom(i);a1;a1 = mol.NextAtom(i))
-          if (static_cast<long int>(a1->GetResidue()->
+        for (OBAtom *a1 = mol.BeginAtom(i);a1;a1 = mol.NextAtom(i)) {
+          // again, atoms may not have residues, but if they do, check serials
+          if (a1->GetResidue() != NULL &&
+              static_cast<long int>(a1->GetResidue()->
                                     GetSerialNum(a1)) == boundedAtomsSerialNumbers[k])
             {
               connectedAtom = a1;
               break;
             }
+        }
         if (connectedAtom == 0L)
           {
             errorMsg << "WARNING: Problems reading a PDB file:\n"
@@ -525,7 +538,7 @@ namespace OpenBabel
                      << "  However, an atom with serial #" << boundedAtomsSerialNumbers[k] << " was not found.\n"
                      << "  THIS CONECT RECORD WILL BE IGNORED." << endl;
             obErrorLog.ThrowError(__FUNCTION__, errorMsg.str() , obWarning);
-	    return(false);
+            return(false);
           }
 
         // Figure the bond order
@@ -605,90 +618,90 @@ namespace OpenBabel
 
         //two char. elements are on position 13 and 14 one char. start at 14
         if (strlen(type_name) > 1)
-	  type_name[1] = toupper(type_name[1]);
+          type_name[1] = toupper(type_name[1]);
         else
-	  {
+          {
             char tmp[10];
             strncpy(tmp, type_name, 10);
             snprintf(type_name, sizeof(type_name), " %-3s", tmp);
-	  }
+          }
 
         if ( (res = atom->GetResidue()) )
-	  {
+          {
             het = res->IsHetAtom(atom);
             snprintf(the_res,4,"%s",(char*)res->GetName().c_str());
             snprintf(type_name,5,"%s",(char*)res->GetAtomID(atom).c_str());
 
             //two char. elements are on position 13 and 14 one char. start at 14
             if (strlen(etab.GetSymbol(atom->GetAtomicNum())) == 1)
-	      {
+              {
                 if (strlen(type_name) < 4)
-		  {
-                    char tmp[10];
-                    strcpy(tmp, type_name);
+                  {
+                    char tmp[16];
+                    strncpy(tmp, type_name, 16);
                     snprintf(padded_name, sizeof(padded_name), " %-3s", tmp);
                     strncpy(type_name,padded_name,4);
                     type_name[4] = '\0';
-		  }
+                  }
                 else
-		  {
+                  {
                     type_name[4] = type_name[3];
                     type_name[3] = type_name[2];
                     type_name[2] = type_name[1];
                     type_name[1] = type_name[0];
                     type_name[0] = type_name[4];
                     type_name[4] = '\0';
-		  }
-	      }
+                  }
+              }
             res_num = res->GetNum();
-	  }
+          }
         else
-	  {
+          {
             strcpy(the_res,"UNK");
             snprintf(padded_name,sizeof(padded_name), "%s",type_name);
             strncpy(type_name,padded_name,4);
             type_name[4] = '\0';
             res_num = 1;
-	  }
+          }
 
         element_name = etab.GetSymbol(atom->GetAtomicNum());
         if (strlen(element_name) == 2)
-	  element_name[1] = toupper(element_name[1]);
+          element_name[1] = toupper(element_name[1]);
         snprintf(buffer, BUFF_SIZE, "%s%5d %-4s %-3s  %4d    %8.3f%8.3f%8.3f  1.00  0.00          %2s  \n",
-                het?"HETATM":"ATOM  ",
-                i,
-                type_name,
-                the_res,
-                res_num,
-                atom->GetX(),
-                atom->GetY(),
-                atom->GetZ(),
-                element_name);
+                 het?"HETATM":"ATOM  ",
+                 i,
+                 type_name,
+                 the_res,
+                 res_num,
+                 atom->GetX(),
+                 atom->GetY(),
+                 atom->GetZ(),
+                 element_name);
         ofs << buffer;
       }
 
     OBAtom *nbr;
     int count;
-    vector<OBEdgeBase*>::iterator k;
+    vector<OBBond*>::iterator k;
     for (i = 1; i <= mol.NumAtoms(); i ++)
       {
         atom = mol.GetAtom(i);
         if (atom->GetValence() <= 4)
-	  {
+          {
             snprintf(buffer, BUFF_SIZE, "CONECT%5d", i);
             ofs << buffer;
             for (nbr = atom->BeginNbrAtom(k);nbr;nbr = atom->NextNbrAtom(k))
-	      {
+              {
                 snprintf(buffer, BUFF_SIZE, "%5d", nbr->GetIdx());
                 ofs << buffer;
-	      }
+              }
             for (count = 0; count < (4 - (int)atom->GetValence()); count++)
-	      {
+              {
                 snprintf(buffer, BUFF_SIZE, "     ");
                 ofs << buffer;
-	      }
+              }
             ofs << "                                       " << endl;
-	  }
+          }
       }
     snprintf(buffer, BUFF_SIZE, "MASTER        0    0    0    0    0    0    0    0 ");
     ofs << buffer;
