@@ -12,61 +12,63 @@ but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 ***********************************************************************/
+#include <openbabel/babelconfig.h>
 
-#include "mol.h"
-#include "obconversion.h"
-#include "math/matrix3x3.h"
-#include "obmolecformat.h"
+#include <openbabel/math/matrix3x3.h>
+#include <openbabel/obmolecformat.h>
 
 using namespace std;
 namespace OpenBabel
 {
 
-class ShelXFormat : public OBMoleculeFormat
-{
-public:
+  class ShelXFormat : public OBMoleculeFormat
+  {
+  public:
     //Register this format type ID
     ShelXFormat()
     {
-        OBConversion::RegisterFormat("res",this);
-        OBConversion::RegisterFormat("ins",this);
+      OBConversion::RegisterFormat("res",this, "chemical/x-shelx");
+      OBConversion::RegisterFormat("ins",this, "chemical/x-shelx");
     }
 
-  virtual const char* Description() //required
-  {
-    return
-      "ShelX format\n \
+    virtual const char* Description() //required
+    {
+      return
+        "ShelX format\n \
        Read Options e.g. -as\n\
         s  Output single bonds only\n\
         b  Disable bonding entirely\n\n";
-  };
+    };
 
-  virtual const char* SpecificationURL()
-  {return "http://shelx.uni-ac.gwdg.de/SHELX/";}; //optional
+    virtual const char* SpecificationURL()
+    {return "http://shelx.uni-ac.gwdg.de/SHELX/";}; //optional
+
+    virtual const char* GetMIMEType() 
+    { return "chemical/x-shelx"; };
 
     //Flags() can return be any the following combined by | or be omitted if none apply
     // NOTREADABLE  READONEONLY  NOTWRITABLE  WRITEONEONLY
     virtual unsigned int Flags()
     {
-        return NOTWRITABLE;
+      return NOTWRITABLE;
     };
 
     ////////////////////////////////////////////////////
     /// The "API" interface functions
     virtual bool ReadMolecule(OBBase* pOb, OBConversion* pConv);
 
-};
+  };
 
-//Make an instance of the format class
-ShelXFormat theShelXFormat;
+  //Make an instance of the format class
+  ShelXFormat theShelXFormat;
 
-/////////////////////////////////////////////////////////////////
-bool ShelXFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
-{
+  /////////////////////////////////////////////////////////////////
+  bool ShelXFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
+  {
 
-    OBMol* pmol = dynamic_cast<OBMol*>(pOb);
+    OBMol* pmol = pOb->CastAndClear<OBMol>();
     if(pmol==NULL)
-        return false;
+      return false;
 
     //Define some references so we can use the old parameter names
     istream &ifs = *pConv->GetInStream();
@@ -81,14 +83,14 @@ bool ShelXFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
     ifs.getline(buffer,BUFF_SIZE);
     mol.SetTitle(buffer);
     while (ifs.getline(buffer,BUFF_SIZE) &&!EQn(buffer,"CELL",4))
-        ;
+      ;
 
     if (!EQn(buffer,"CELL",4))
-        return(false);
+      return(false);
     vector<string> vs;
     tokenize(vs,buffer," \n\t,");
     if (vs.size() != 8)
-        return(false);
+      return(false);
 
     //parse cell values
     A = atof((char*)vs[2].c_str());
@@ -98,6 +100,7 @@ bool ShelXFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
     Beta  = atof((char*)vs[6].c_str());
     Gamma = atof((char*)vs[7].c_str());
     OBUnitCell *uc = new OBUnitCell;
+    uc->SetOrigin(fileformatInput);
     uc->SetData(A, B, C, Alpha, Beta, Gamma);
     mol.SetData(uc);
     m = uc->GetOrthoMatrix();
@@ -110,18 +113,18 @@ bool ShelXFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
 
     //skip until FVAR found
     while (ifs.getline(buffer,BUFF_SIZE) &&!EQn(buffer,"FVAR",4))
-        ;
+      ;
 
     mol.BeginModify();
 
     //read atom records
     while (ifs.getline(buffer,BUFF_SIZE) &&!EQn(buffer,"HKLF",4))
-    {
+      {
         tokenize(vs,buffer," \n\t,");
 
         //skip AFIX and PART instructions
         if (vs.size() < 7)
-            continue;
+          continue;
         atom = mol.NewAtom();
 
         x = atof((char*)vs[2].c_str());
@@ -139,8 +142,8 @@ bool ShelXFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
 
         //skip next line if anisotropic atoms.
         if (vs.size() == 9)
-            ifs.getline(buffer,BUFF_SIZE);
-    } //while
+          ifs.getline(buffer,BUFF_SIZE);
+      } //while
 
     if (!pConv->IsOption("b",OBConversion::INOPTIONS))
       mol.ConnectTheDots();
@@ -149,6 +152,6 @@ bool ShelXFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
 
     mol.EndModify();
     return(true);
-}
+  }
 
 } //namespace OpenBabel
