@@ -13,9 +13,8 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 ***********************************************************************/
 
-#include "mol.h"
-#include "obconversion.h"
-#include "obmolecformat.h"
+#include <openbabel/babelconfig.h>
+#include <openbabel/obmolecformat.h>
 
 #include <sstream>
 
@@ -35,14 +34,14 @@ namespace OpenBabel
     virtual const char* Description() //required
     {
       return
-        "XYZ cartesian coordinates format\n \
-       Read Options e.g. -as\n\
-        s  Output single bonds only\n\
-        b  Disable bonding entirely\n\n";
+        "XYZ cartesian coordinates format\n"
+        "Read Options e.g. -as\n"
+        "  s  Output single bonds only\n"
+        "  b  Disable bonding entirely\n\n";
     };
 
     virtual const char* SpecificationURL()
-    {return "http://openbabel.sourceforge.net/formats/xyz.shtml";}; //optional
+    {return "http://openbabel.sourceforge.net/wiki/XYZ";}; //optional
 
     virtual const char* GetMIMEType() 
     { return "chemical/x-xyz"; };
@@ -61,7 +60,7 @@ namespace OpenBabel
   /////////////////////////////////////////////////////////////////
   bool XYZFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
   {
-    OBMol* pmol = dynamic_cast<OBMol*>(pOb);
+    OBMol* pmol = pOb->CastAndClear<OBMol>();
     if(pmol==NULL)
       return false;
 
@@ -69,7 +68,6 @@ namespace OpenBabel
     istream &ifs = *pConv->GetInStream();
     OBMol &mol = *pmol;
     const char* title = pConv->GetTitle();
-
     char buffer[BUFF_SIZE];
 
     stringstream errorMsg;
@@ -224,29 +222,25 @@ namespace OpenBabel
     ostream &ofs = *pConv->GetOutStream();
     OBMol &mol = *pmol;
 
-    unsigned int i;
     char buffer[BUFF_SIZE];
 
-    snprintf(buffer, BUFF_SIZE, "%d", mol.NumAtoms());
-    ofs << buffer << endl;
+    snprintf(buffer, BUFF_SIZE, "%d\n", mol.NumAtoms());
+    ofs << buffer;
     if (fabs(mol.GetEnergy()) > 1.0e-3) // nonzero energy field
-      snprintf(buffer, BUFF_SIZE, "%s\tEnergy: %15.7f",
+      snprintf(buffer, BUFF_SIZE, "%s\tEnergy: %15.7f\n",
                mol.GetTitle(), mol.GetEnergy());
     else
-      snprintf(buffer, BUFF_SIZE, "%s", mol.GetTitle());
-    ofs << buffer << endl;
+      snprintf(buffer, BUFF_SIZE, "%s\n", mol.GetTitle());
+    ofs << buffer;
 
-    OBAtom *atom;
-    string str,str1;
-    for(i = 1;i <= mol.NumAtoms(); i++)
+    FOR_ATOMS_OF_MOL(atom, mol)
       {
-        atom = mol.GetAtom(i);
-        snprintf(buffer, BUFF_SIZE, "%3s%15.5f%15.5f%15.5f",
+        snprintf(buffer, BUFF_SIZE, "%-3s%15.5f%15.5f%15.5f\n",
                  etab.GetSymbol(atom->GetAtomicNum()),
                  atom->GetX(),
                  atom->GetY(),
                  atom->GetZ());
-        ofs << buffer << endl;
+        ofs << buffer;
       }
 
     return(true);
