@@ -9,7 +9,7 @@
  **********************************************************
  *   Reader and Writer for Yasara Objects (*.YOB files)   *
  **********************************************************
- *            (C) 2002-2005 by Elmar Krieger              *
+ *            (C) 2002-2006 by Elmar Krieger              *
  **********************************************************
 
 This reader/writer for Yasara objects is free software;
@@ -23,6 +23,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 ***********************************************************/
+#include <openbabel/babelconfig.h>
 
 /* Adapt on some 64bit machines! */
 typedef int int32;
@@ -30,9 +31,7 @@ typedef unsigned int uint32;
 typedef short int16;
 typedef unsigned short uint16;
 
-#include "mol.h"
-#include "obconversion.h"
-#include "obmolecformat.h"
+#include <openbabel/obmolecformat.h>
 
 #define mem_alloc malloc
 #define mem_free free
@@ -55,7 +54,7 @@ int int32lemem(char *data)
   value=(int)((unsigned char*)data)[0]+((int)((unsigned char*)data)[1]<<8)+
         ((int)((unsigned char*)data)[2]<<16)+((int)((unsigned char*)data)[3]<<24);
   intmino2=-1073741824;
-  if (INT_MIN!=0x80000000&&(value&0x80000000)) value=(intmino2*2)+(value&0x7fffffff);
+  if ((unsigned int) INT_MIN!=0x80000000&&(value&0x80000000)) value=(intmino2*2)+(value&0x7fffffff);
   return(value); }
 
 int int32le(int32 value)
@@ -353,24 +352,24 @@ YOBFormat theYOBFormat;
    IF options CONTAINS 'f' (COMMAND LINE -af), ATOM NAMES WILL STAY FIXED (I.E. LEAVING THE SPACE IN THE FIRST COLUMN) */
 bool YOBFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
 { 
-  OBMol* pmol = dynamic_cast<OBMol*>(pOb);
+  OBMol* pmol = pOb->CastAndClear<OBMol>();
   if(pmol==NULL)
       return false;
 
   //Define some references so we can use the old parameter names
   istream &ifs = *pConv->GetInStream();
   OBMol &mol = *pmol;
-  const char* title = pConv->GetTitle();
+  //   const char* title = pConv->GetTitle();
 
   bool hetatom;
   char buffer[8],resname[4],atomname[5];
   char *str;
-  int i,j,m,q;
-  int resno,chainNum,link,linked,linktype,atoms,element,links,chain;
-  int samenames,reslen,charged;
+  unsigned int i,j/*,m,q*/;
+  unsigned int /*resno,chainNum,*/link,linked,linktype,atoms,element,links,chain;
+  int /*samenames,*/reslen,charged;
   unsigned int infosize,size;
   mobdata *mob;
-  struct mobatom *srcatom,*atom2,*resstart;
+  struct mobatom *srcatom,/**atom2,*/*resstart;
   struct atomid id;
   OBAtom *dstatom;
   OBResidue *res;
@@ -477,22 +476,22 @@ bool YOBFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
   ostream &ofs = *pConv->GetOutStream();
   OBMol &mol = *pmol;
 
-  bool hetatom;
-  char buffer[32],resname[4],atomname[5];
+  //  bool hetatom;
+  char buffer[32],/*resname[4],*/atomname[5];
   char double1[8]={0,0,0,0,0,0,-16,0x3f};
-  char *str;
-  int i,j,m,q,pos;
-  int resno,chainNum,link,linktype,atoms,element,links,chain;
-  int samenames,reslen,bondorder,flags;
-  unsigned int infosize,size;
-  mobdata *mob;
-  struct mobatom *atom2,*resstart;
-  struct atomid id;
+  //   char *str;
+  int i,j,/*m,q,*/pos;
+  int /*resno,chainNum,link,linktype,*/atoms,element,links/*,chain*/;
+  int /*samenames,reslen,*/bondorder,flags;
+  unsigned int /*infosize,*/size;
+  //   mobdata *mob;
+  //   struct mobatom *atom2,*resstart;
+  //   struct atomid id;
   OBAtom *srcatom,*linkedatom;
   OBResidue *res;
   OBBond *bond;
-  vector<OBNodeBase*>::iterator e;
-  vector<OBEdgeBase*>::iterator iter;
+  vector<OBAtom*>::iterator e;
+  vector<OBBond*>::iterator iter;
 
   atoms=mol.NumAtoms();
   if (!atoms) return(false);
