@@ -2,7 +2,7 @@
 obconversion.cpp -  Declaration of OBFormat and OBConversion
 
 Copyright (C) 2004 by Chris Morley
-Some portions Copyright (C) 2005 by Geoffrey Hutchison
+Some portions Copyright (C) 2005-2006 by Geoffrey Hutchison
 
 This file is part of the Open Babel project.
 For more information, see <http://openbabel.sourceforge.net/>
@@ -17,27 +17,28 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 ***********************************************************************/
 // Definition of OBConversion routines
+#include <openbabel/babelconfig.h>
 
 #ifdef _WIN32
-        #pragma warning (disable : 4786)
+	#pragma warning (disable : 4786)
 
-        //using 'this' in base class initializer
-        #pragma warning (disable : 4355)
+	//using 'this' in base class initializer
+	#pragma warning (disable : 4355)
 
-        #ifdef GUI
-                #undef DATADIR
-                #include "stdafx.h" //(includes<windows.h>
-        #endif
+	#ifdef GUI
+		#undef DATADIR
+		#include "stdafx.h" //(includes<windows.h>
+	#endif
 #endif
 
-#include <iostream>
+#include <iosfwd>
 #include <fstream>
 #include <sstream>
 #include <string>
 #include <map>
-//#include <dlfcn.h>
 
-#include "obconversion.h"
+#include <openbabel/obconversion.h>
+#include "openbabel/lineend.h"
 
 #ifdef HAVE_LIBZ
 #include "zipstream.h"
@@ -52,8 +53,28 @@ extern "C" int strncasecmp(const char *s1, const char *s2, size_t n);
 #endif
 
 using namespace std;
+//using namespace boost::iostreams;
+
 namespace OpenBabel {
 
+  /** @class OBFormat obconversion.h <openbabel/obconversion.h>
+      Two sets of Read and Write functions are specified for each format
+      to handle two different requirements.
+      The "Convert" interface is for use in file format conversion applications. The
+      user interface, a console, a GUI, or another program is kept unaware of the
+      details of the chemistry and does not need to \#include mol.h. It is then
+      necessary to manipulate only pointers to OBBase in OBConversion and the user
+      interface, with all the construction and deletion of OBMol etc objects being
+      done in the Format classes or the OB core. The convention  with "Covert"
+      interface functions is that chemical objects are made on the heap with new
+      in the ReadChemicalObject() functions and and deleted in WriteChemicalObject()
+      functions
+
+      The "API" interface is for programatic use of the OB routines in application
+      programs where mol.h is \#included. There is generally no creation or
+      destruction of objects in ReadMolecule() and WriteMolecule() and no restriction
+      on whether the pointers are to the heap or the stack.
+  **/
   const char* OBFormat::TargetClassDescription()
   {
     //Provides class of default format unless overridden
@@ -61,7 +82,7 @@ namespace OpenBabel {
       return OBConversion::GetDefaultFormat()->TargetClassDescription();
     else
       return "";
-  };
+  }
   const type_info& OBFormat::GetType()
   {
     //Provides info on class of default format unless overridden
@@ -69,11 +90,11 @@ namespace OpenBabel {
       return OBConversion::GetDefaultFormat()->GetType();
     else
       return typeid(this); //rubbish return if DefaultFormat not set
-  };
+  }
 
   //***************************************************
 
-  /** @class OBConversion
+  /** @class OBConversion obconversion.h <openbabel/obconversion.h>
       OBConversion maintains a list of the available formats, 
       provides information on them, and controls the conversion process.
 
@@ -117,11 +138,11 @@ namespace OpenBabel {
       OBConversion conv(&cin,&cout);
       if(conv.SetInAndOutFormats("SMI","MOL"))
       {	
-      OBMol mol;
-      if(conv.Read(&mol))
-      ...manipulate molecule 
+         OBMol mol;
+         if(conv.Read(&mol))
+            // ...manipulate molecule 
 		
-      conv->Write(&mol);
+         conv->Write(&mol);
       }
       @endcode
 	
@@ -139,19 +160,19 @@ namespace OpenBabel {
       stringstream ss(SmilesString)
       OBConversion conv(&ss);
       if(conv.SetInFormat("smi") && conv.Read(&mol))
-      ...
+         // ...
       @endcode
 
       <b>To do a file conversion without manipulating the molecule.</b>
 
       @code
-      #include "obconversion.h" //mol.h is not needed
+      #include <openbabel/obconversion.h> //mol.h is not needed
       ...set up an istream is and an ostream os 
       OBConversion conv(&is,&os);
       if(conv.SetInAndOutFormats("SMI","MOL"))
       {
-      conv.SetOptions("h"); //Optional; (h adds expicit hydrogens)
-      conv.Convert();
+         conv.SetOptions("h"); //Optional; (h adds expicit hydrogens)
+         conv.Convert();
       }
       @endcode
 
@@ -172,9 +193,9 @@ namespace OpenBabel {
       stringstream newstream;
       if(inFormat && outFormat)
       {
-      conv.SetInAndOutFormats(inFormat,outFormat);
-      conv.Convert(pIn,&newstream);
-      pIn=&newstream;
+         conv.SetInAndOutFormats(inFormat,outFormat);
+         conv.Convert(pIn,&newstream);
+         pIn=&newstream;
       }
       //else error; new features not available; fallback to original functionality 
 
@@ -193,8 +214,10 @@ namespace OpenBabel {
 
   OBConversion::OBConversion(istream* is, ostream* os) : 
     pInFormat(NULL),pOutFormat(NULL), Index(0), StartNumber(1),
-    EndNumber(0), Count(-1), m_IsLast(true), MoreFilesToCome(false),
-    OneObjectOnly(false), pOb1(NULL), pAuxConv(NULL)
+    EndNumber(0), Count(-1), m_IsFirstInput(true), m_IsLast(true),
+    MoreFilesToCome(false), OneObjectOnly(false), CheckedForGzip(false),
+    NeedToFreeInStream(false), NeedToFreeOutStream(false), 
+    pOb1(NULL), pAuxConv(NULL)
   {
     pInStream=is;
     pOutStream=os;
@@ -212,9 +235,7 @@ namespace OpenBabel {
   ///See Marshall Cline's C++ FAQ Lite document, www.parashift.com/c++-faq-lite/". 
   FMapType& OBConversion::FormatsMap()
   {
-    static FMapType* fm = NULL;
-    if (!fm)
-      fm = new FMapType;
+    static FMapType* fm = new FMapType;
     return *fm;
   }
 
@@ -224,9 +245,7 @@ namespace OpenBabel {
   ///See Marshall Cline's C++ FAQ Lite document, www.parashift.com/c++-faq-lite/". 
   FMapType& OBConversion::FormatsMIMEMap()
   {
-    static FMapType* fm = NULL;
-    if (!fm)
-      fm = new FMapType;
+    static FMapType* fm = new FMapType;
     return *fm;
   }
 
@@ -254,15 +273,28 @@ namespace OpenBabel {
     OneObjectOnly  = o.OneObjectOnly;
     pOb1           = o.pOb1;
     ReadyToInput   = o.ReadyToInput;
-	
+    m_IsFirstInput = o.m_IsFirstInput;
+    CheckedForGzip = o.CheckedForGzip;
+    NeedToFreeInStream = o.NeedToFreeInStream;
+    NeedToFreeOutStream = o.NeedToFreeOutStream;
+
     pAuxConv       = NULL;
   }
-  ////////////////////////////////////////////////
+  ///////////////////////////////////////////////
 
   OBConversion::~OBConversion() 
   {
     if(pAuxConv!=this)
       delete pAuxConv;
+    // Free any remaining streams from convenience functions
+    if(pInStream && NeedToFreeInStream) {
+      delete pInStream;
+      NeedToFreeInStream = false;
+    }
+    if(pOutStream && NeedToFreeOutStream) {
+      delete pOutStream;
+      NeedToFreeOutStream = false;
+    }
   }
   //////////////////////////////////////////////////////
 
@@ -302,12 +334,12 @@ namespace OpenBabel {
     if(!DLHandler::findFiles(files,DLHandler::getFormatFilePattern(),TargetDir)) return 0;
 
     vector<string>::iterator itr;
-    for(itr=files.begin();itr!=files.end();itr++)
+    for(itr=files.begin();itr!=files.end();++itr)
       {
         if(DLHandler::openLib(*itr))
           count++;
         else
-          cerr << *itr << " did not load properly" << endl;
+          obErrorLog.ThrowError(__FUNCTION__, *itr + " did not load properly", obError);
       }
 #else
     count = 1; //avoid calling this function several times
@@ -325,10 +357,10 @@ namespace OpenBabel {
    *@code
    *	const char* str=NULL;
    *	Formatpos pos;
-   *     OBConversion conv; // dummy to make sure static data is available
+   *	OBConversion conv; // dummy to make sure static data is available
    *	while(OBConversion::GetNextFormat(pos,str,pFormat))
    *	{
-   *		use str and pFormat
+   *		// use str and pFormat
    *	}
    *@endcode
    *
@@ -413,14 +445,26 @@ namespace OpenBabel {
   //////////////////////////////////////////////////////
   int OBConversion::Convert(istream* is, ostream* os) 
   {
-    if(is) pInStream=is;
-    if(os) pOutStream=os;
+    if (is) { 
+      pInStream=is;
+      CheckedForGzip = false; // haven't checked this for gzip yet
+    }
+    if (os) pOutStream=os;
     ostream* pOrigOutStream = pOutStream;
 
 #ifdef HAVE_LIBZ
-    zlib_stream::zip_istream zIn(*pInStream);
-    if(zIn.is_gzip())
-      pInStream = &zIn;
+    zlib_stream::zip_istream *zIn;
+
+    // only try to decode the gzip stream once
+    if (!CheckedForGzip) {
+      zIn = new zlib_stream::zip_istream(*pInStream);
+      if (zIn->is_gzip()) {
+        pInStream = zIn;
+        CheckedForGzip = true;
+      }
+      else
+        delete zIn;
+    }
 
     zlib_stream::zip_ostream zOut(*pOutStream);
     if(IsOption("z",GENOPTIONS))
@@ -431,12 +475,30 @@ namespace OpenBabel {
       }
 #endif
 
+    //The FilteringInputStreambuf delivers characters to the istream, pInStream,
+    //and receives characters this stream's original rdbuf. 
+    //It filters them, converting CRLF and CR line endings to LF.
+    //seek and tellg requests to the stream are passed through to the original
+    //rdbuf.
+    //streambuf objects cannot be member variables and so need to be made here.
+    //Because FilteringInputStreambuf has to be at this level to remain
+    //in scope during Convert(), it is always constructed, but is only used in
+    //with formats with non-binary and which are not XML (which should not be 
+    //sensitive to line endings anyway).
+
+    FilteringInputStreambuf< LineEndingExtractor > LineEndBuf(pInStream->rdbuf());
+    streambuf* pOrigInBuf = pInStream->rdbuf();
+    if(pInFormat && !(pInFormat->Flags() & READBINARY) && !(pInFormat->Flags() & READXML))
+      pOrigInBuf = pInStream->rdbuf(&LineEndBuf);
+    
     int count = Convert();
+
+    pInStream->rdbuf(pOrigInBuf);
     pOutStream = pOrigOutStream;
     return count;
-
   }
 
+
   ////////////////////////////////////////////////////
   /// Actions the "convert" interface.
   ///	Calls the OBFormat class's ReadMolecule() which 
@@ -459,7 +521,7 @@ namespace OpenBabel {
   {
     if(pInStream==NULL || pOutStream==NULL)
       {
-        cerr << "input or output stream not set" << endl;
+        obErrorLog.ThrowError(__FUNCTION__, "input or output stream not set", obError);
         return 0;
       }
 
@@ -475,7 +537,7 @@ namespace OpenBabel {
     wInlen=0;
 
     //Input loop
-    while(ReadyToInput && pInStream->peek() != EOF && pInStream->good())
+    while(ReadyToInput && pInStream->good()) //Possible to omit? && pInStream->peek() != EOF 
       {
         if(pInStream==&cin)
           {
@@ -489,11 +551,12 @@ namespace OpenBabel {
         try
           {
             ret = pInFormat->ReadChemObject(this);
+            m_IsFirstInput=false;
           }		
         catch(...)
           {
             if(!IsOption("e", GENOPTIONS) && !OneObjectOnly)
-              throw;
+              return Index; // the number we've actually output so far
           }
 
         if(!ret)
@@ -511,12 +574,11 @@ namespace OpenBabel {
       }
 	
     //Output last object
-    //if(!MoreFilesToCome)
-    //	m_IsLast=true;
     m_IsLast= !MoreFilesToCome;
 
-    if(pOutFormat)
-      if(!pOutFormat->WriteChemObject(this))
+    //Output is deferred until the end with -C option
+    if(pOutFormat && (!IsOption("C",GENOPTIONS) || m_IsLast))
+      if(pOb1 && !pOutFormat->WriteChemObject(this))
         Index--;
 	
     //Put AddChemObject() into non-queue mode
@@ -530,7 +592,7 @@ namespace OpenBabel {
   //////////////////////////////////////////////////////
   bool OBConversion::SetStartAndEnd()
   {
-    int TempStartNumber=0;
+    unsigned int TempStartNumber=0;
     const char* p = IsOption("f",GENOPTIONS);
     if(p)
       {
@@ -581,14 +643,13 @@ namespace OpenBabel {
   ///    Count is incremented with each call, even if pOb=NULL. 
   ///    Objects are not added to the queue if the count is outside the range  
   ///    StartNumber to EndNumber. There is no upper limit if EndNumber is zero. 
-  ///    The return value is the number of objects, including this one, which have been
-  ///    input (but not necessarily output).
-  int OBConversion::AddChemObject(OBBase* pOb)
+  ///    The return value is the return code from WriteChemObject - false on error.
+  bool OBConversion::AddChemObject(OBBase* pOb)
   {
     if(Count<0) 
       {
         pOb1=pOb;
-        return Count;
+        return Count!=0;
       }
     Count++;
     if(Count>=(int)StartNumber)//keeps reading objects but does nothing with them
@@ -607,29 +668,32 @@ namespace OpenBabel {
                   {
                     //faultly write, so finish
                     --Index;
-                    ReadyToInput=false;
-                    return Count;
+                    //ReadyToInput=false;
+                    pOb1=NULL;
+                    return false;
                   }
                 //Stop after writing with single object output files
                 if(pOutFormat->Flags() & WRITEONEONLY)
                   {
+                    // if there are more molecules to output, send a warning
+                    stringstream errorMsg;
+                    errorMsg << "WARNING: You are attempting to convert a file"
+                             << " with multiple molecule entries into a format"
+                             << " which can only store one molecule. The current"
+                             << " output will only contain the first molecule.\n\n";
+
+                    errorMsg << "To convert this input into multiple separate"
+                             << " output files, with one molecule per file, try:\n"
+                             << "babel [input] [ouptut] -m\n\n";
+
+                    errorMsg << "To pick one particular molecule"
+                             << " (e.g., molecule 4), try:\n"
+                             << "babel -f 4 -l 4 [input] [output]" << endl;
+
+                    obErrorLog.ThrowError(__FUNCTION__, errorMsg.str(), obWarning);
+
                     ReadyToInput = false;
                     pOb1 = NULL;
-
-                    // if there are more molecules to output, send a warning
-                    cerr << "WARNING: You are attempting to convert a file"
-                         << " with multiple molecule entries into a format"
-                         << " which can only store one molecule. The current"
-                         << " output will only contain the first molecule.\n\n";
-
-                    cerr << "To convert this input into multiple separate"
-                         << " output files, with one molecule per file, try:\n"
-                         << "babel [input] [ouptut] -m\n\n";
-
-                    cerr << "To pick one particular molecule"
-                         << " (e.g., molecule 4), try:\n"
-                         << "babel -f 4 -l 4 [input] [output]" << endl;
-
                     return true;
                   }
               }
@@ -638,9 +702,12 @@ namespace OpenBabel {
             wInlen = rInlen;
           }
       }
-    return Count;
+    return true;
   }
   //////////////////////////////////////////////////////
+  ///Returns the number of objects which have been output or are currently being output.
+  ///The outputindex is incremented when an object for output is fetched by GetChemObject().
+  ///So the function will return 1 if called from WriteMolecule() during output of the first object.
   int OBConversion::GetOutputIndex() const
   {
     //The number of objects actually written already from this instance of OBConversion
@@ -671,17 +738,17 @@ namespace OpenBabel {
     MoreFilesToCome=true;
   }
 
-  void OBConversion::SetOneObjectOnly()
+  void OBConversion::SetOneObjectOnly(bool b)
   {
-    OneObjectOnly=true;
-    m_IsLast=true;
+    OneObjectOnly=b;
+    m_IsLast=b;
   }	
 
   /////////////////////////////////////////////////////////
   OBFormat* OBConversion::FormatFromExt(const char* filename)
   {
     string file = filename;
-    size_t extPos = file.rfind(".");
+    string::size_type extPos = file.rfind('.');
 
     if(extPos!=string::npos)
       {
@@ -690,7 +757,7 @@ namespace OpenBabel {
         if (file.substr(extPos,3) == ".gz")
           {
             file.erase(extPos);
-            extPos = file.rfind(".");
+            extPos = file.rfind('.');
             if (extPos!=string::npos)
               return FindFormat( (file.substr(extPos + 1, file.size())).c_str() );
           }
@@ -711,27 +778,47 @@ namespace OpenBabel {
 
   bool	OBConversion::Read(OBBase* pOb, std::istream* pin)
   {
-    if(pin)
+    if(pin) { 
       pInStream=pin;
-    if(!pInFormat) return false;
+      CheckedForGzip = false; // haven't set this stream to gzip (yet)
+    }
+
+    if(!pInFormat || !pInStream) return false;
 
 #ifdef HAVE_LIBZ
-    zlib_stream::zip_istream zIn(*pInStream);
-    if(zIn.is_gzip())
-      pInStream = &zIn;
+    zlib_stream::zip_istream *zIn;
+
+    // only try to decode the gzip stream once
+    if (!CheckedForGzip) {
+      zIn = new zlib_stream::zip_istream(*pInStream);
+      if (zIn->is_gzip()) {
+        pInStream = zIn;
+        CheckedForGzip = true;
+      }
+      else
+        delete zIn;
+    }
 #endif
 
+    FilteringInputStreambuf< LineEndingExtractor > LineEndBuf(pInStream->rdbuf());
+    streambuf* pOrigInBuf = pInStream->rdbuf();
+    if(!(pInFormat->Flags() & READBINARY) && !(pInFormat->Flags() & READXML))
+      pOrigInBuf = pInStream->rdbuf(&LineEndBuf);
+
+    pInStream->rdbuf(pOrigInBuf);
+
     return pInFormat->ReadMolecule(pOb, this);
   }
+
   //////////////////////////////////////////////////
   /// Writes the object pOb but does not delete it afterwards.
   /// The output stream is lastingly changed if pos is not NULL
   /// Returns true if successful.
   bool OBConversion::Write(OBBase* pOb, ostream* pos)
   {
-    if(pos)
-      pOutStream=pos;
-    if(!pOutFormat) return false;
+    if(pos) pOutStream=pos;
+
+    if(!pOutFormat || !pOutStream) return false;
 
     ostream* pOrigOutStream = pOutStream;
 #ifdef HAVE_LIBZ
@@ -755,10 +842,11 @@ namespace OpenBabel {
   /// Writes the object pOb but does not delete it afterwards.
   /// The output stream not changed (since we cannot write to this string later)
   /// Returns true if successful.
-  std::string OBConversion::WriteString(OBBase* pOb)
+  std::string OBConversion::WriteString(OBBase* pOb, bool trimWhitespace)
   {
     ostream *oldStream = pOutStream; // save old output
     stringstream newStream;
+    string temp;
 
     if(pOutFormat)
       {
@@ -766,7 +854,13 @@ namespace OpenBabel {
       }
     pOutStream = oldStream;
 
-    return newStream.str();
+    temp = newStream.str();
+    if (trimWhitespace) // trim the trailing whitespace
+      {
+        string::size_type notwhite = temp.find_last_not_of(" \t\n\r");
+        temp.erase(notwhite+1);
+      }
+    return temp;
   }
 
   //////////////////////////////////////////////////
@@ -777,25 +871,47 @@ namespace OpenBabel {
   {
     if(!pOutFormat) return false;
 
+    // if we have an old stream, free this first before creating a new one
+    if (pOutStream && NeedToFreeOutStream) {
+      delete pOutStream;
+    }
+
     ofstream *ofs = new ofstream;
+    NeedToFreeOutStream = true; // make sure we clean this up later
     ios_base::openmode omode = 
       pOutFormat->Flags() & WRITEBINARY ? ios_base::out|ios_base::binary : ios_base::out;
 
     ofs->open(filePath.c_str(),omode);
     if(!ofs || !ofs->good())
       {
-        cerr << "Cannot write to " << filePath <<endl;
+        obErrorLog.ThrowError(__FUNCTION__,"Cannot write to " + filePath, obError);
         return false;
       }
 
     return Write(pOb, ofs);
   }
 
+  void OBConversion::CloseOutFile()
+  {
+    if (pOutStream && NeedToFreeOutStream)
+    {
+      delete pOutStream;
+      NeedToFreeOutStream = false;
+      pOutStream = NULL;
+    }
+  }
+
   ////////////////////////////////////////////
   bool	OBConversion::ReadString(OBBase* pOb, std::string input)
   {
+    // if we have an old stream, free this first before creating a new one
+    if (pInStream && NeedToFreeInStream) {
+      delete pInStream;
+    }
+
     stringstream *pin = new stringstream(input);
-    return Read(pOb,pin);
+    NeedToFreeInStream = true; // make sure we clean this up later
+    return Read(pOb, pin);
   }
 
 
@@ -804,14 +920,20 @@ namespace OpenBabel {
   {
     if(!pInFormat) return false;
 
+    // if we have an old stream, free this first before creating a new one
+    if (pInStream && NeedToFreeInStream) {
+      delete pInStream;
+    }
+
     ifstream *ifs = new ifstream;
+    NeedToFreeInStream = true; // make sure we free this
     ios_base::openmode imode = 
       pInFormat->Flags() & READBINARY ? ios_base::in|ios_base::binary : ios_base::in;
 
     ifs->open(filePath.c_str(),imode);
     if(!ifs || !ifs->good())
       {
-        cerr << "Cannot read from " << filePath << endl;
+        obErrorLog.ThrowError(__FUNCTION__,"Cannot read from " + filePath, obError);
         return false;
       }
 
@@ -822,12 +944,21 @@ namespace OpenBabel {
   ////////////////////////////////////////////
   const char* OBConversion::Description()
   {
+#ifdef HAVE_LIBZ
     return "Conversion options\n \
  -f <#> Start import at molecule # specified\n \
  -l <#> End import at molecule # specified\n \
- -t All input files describe a single molecule\n \
  -e Continue with next object after error, if possible\n \
- -z Compress the output with gzip\n";
+ -z Compress the output with gzip\n \
+ -k Attempt to translate keywords\n";
+#else
+    return "Conversion options\n \
+ -f <#> Start import at molecule # specified\n \
+ -l <#> End import at molecule # specified\n \
+ -e Continue with next object after error, if possible\n \
+ -k Attempt to translate keywords\n";
+#endif
+    // -t All input files describe a single molecule
   }
 
   ////////////////////////////////////////////
@@ -838,7 +969,7 @@ namespace OpenBabel {
   ////////////////////////////////////////////
   bool OBConversion::IsFirstInput()
   {
-    return (Count==0);
+    return m_IsFirstInput;
   }
 
   /////////////////////////////////////////////////
@@ -865,7 +996,7 @@ namespace OpenBabel {
 #endif
         }
 
-        int posname= InFile.find_last_of("\\/");
+        string::size_type posname= InFile.find_last_of("\\/");
         ofname.replace(pos,1, InFile, posname+1, posdot-posname-1);
       }
     return ofname;	
@@ -876,8 +1007,8 @@ namespace OpenBabel {
   {
     //Replaces * in BaseName by Count
     string ofname(BaseName);
-    int pos = ofname.find('*');
-    if(pos>=0)
+    string::size_type pos = ofname.find('*');
+    if(pos!=string::npos)
       {
         char num[33];
         snprintf(num, 33, "%d", Count);
@@ -886,7 +1017,28 @@ namespace OpenBabel {
     return ofname;		
   }
   ////////////////////////////////////////////////////
+  bool OBConversion::CheckForUnintendedBatch(const string& infile, const string& outfile)
+  {
+    //If infile == outfile issue error message and return false
+    //If name without the extensions are the same issue warning and return true;
+    //Otherwise return true
+    bool ret=true;
+    string inname1, inname2;
+    string::size_type pos;
+    pos = infile.rfind('.');
+    if(pos != string::npos)
+      inname1 = infile.substr(0,pos);
+    pos = outfile.rfind('.');
+    if(pos != string::npos)
+      inname2 = infile.substr(0,pos);
+    if(inname1==inname2) 
+      obErrorLog.ThrowError(__FUNCTION__,
+"This was a batch operation. For splitting, use non-empty base name for the output files", obWarning);
 
+    if(infile==outfile)
+      return false;
+    return true;
+  }
   /**
      Makes input and output streams, and carries out normal,
      batch, aggregation, and splitting conversion.
@@ -904,7 +1056,8 @@ namespace OpenBabel {
      Done if FileList contains a single file name and OutputFileName
      contains a * . Each chemical object in the input file is converted
      and sent to a separate file whose name is OutputFileName with the
-     * replaced by 1, 2, 3, etc.
+     * replaced by 1, 2, 3, etc.  OutputFileName must have at least one
+     character other than the * before the extension.
      For example, if OutputFileName is NEW*.smi then the output files are
      NEW1.smi, NEW2.smi, etc.
 
@@ -928,15 +1081,19 @@ namespace OpenBabel {
   int OBConversion::FullConvert(std::vector<std::string>& FileList, std::string& OutputFileName,
                                 std::vector<std::string>& OutputFileList)
   {
-    ostream* pOs=NULL;
+	
     istream* pIs=NULL;
+    ostream* pOs=NULL;
     ifstream is;
     ofstream os;
+    stringstream ssOut;
     bool HasMultipleOutputFiles=false;
     int Count=0;
+    m_IsFirstInput=true;
     bool CommonInFormat = pInFormat ? true:false; //whether set in calling routine
     ios_base::openmode omode = 
       pOutFormat->Flags() & WRITEBINARY ? ios_base::out|ios_base::binary : ios_base::out;
+    obErrorLog.ClearLog();
     try
       {
         ofstream ofs;
@@ -949,14 +1106,28 @@ namespace OpenBabel {
             if(OutputFileName.find_first_of('*')!=string::npos) HasMultipleOutputFiles = true;
             if(!HasMultipleOutputFiles)
               {
-                os.open(OutputFileName.c_str(),omode);
-                if(!os)
+                //If the output file is the same as any of the input
+                //files, send the output to a temporary stringstream
+                vector<string>::iterator itr;
+                for(itr=FileList.begin();itr!=FileList.end();++itr)
                   {
-                    cerr << "Cannot write to " << OutputFileName <<endl;
-                    return 0;
+                    if(*itr==OutputFileName)
+                      {
+                        pOs = &ssOut;
+                        break;
+                      }
+                  }
+                if(itr==FileList.end())
+                  {
+                    os.open(OutputFileName.c_str(),omode);
+                    if(!os)
+                      {
+                        obErrorLog.ThrowError(__FUNCTION__,"Cannot write to " + OutputFileName, obError);
+                        return 0;
+                      }
+                    pOs=&os;
                   }
                 OutputFileList.push_back(OutputFileName);
-                pOs=&os;
               }
           }
 
@@ -965,18 +1136,19 @@ namespace OpenBabel {
             //Concatenate input file option (multiple files, single molecule)
             if(HasMultipleOutputFiles)
               {
-                cerr << "Cannot have multiple output files and also concatenate input files (-t option)" <<endl;
+                obErrorLog.ThrowError(__FUNCTION__,
+                                      "Cannot have multiple output files and also concatenate input files (-t option)",obError);
                 return 0;
               }
 
             stringstream allinput;
             vector<string>::iterator itr;
-            for(itr=FileList.begin();itr!=FileList.end();itr++)
+            for(itr=FileList.begin();itr!=FileList.end();++itr)
               {
                 ifstream ifs((*itr).c_str());
                 if(!ifs)
                   {
-                    cerr << "Cannot open " << *itr <<endl;
+                    obErrorLog.ThrowError(__FUNCTION__,"Cannot open " + *itr, obError);
                     continue;
                   }
                 allinput << ifs.rdbuf(); //Copy all file contents
@@ -991,13 +1163,13 @@ namespace OpenBabel {
           pIs = NULL;
         else
           {
-            if(FileList.size()>1)
+            if(FileList.size()>1 || OutputFileName.substr(0,2)=="*.")
               {
                 //multiple input files
                 vector<string>::iterator itr, tempitr;
                 tempitr = FileList.end();
                 tempitr--;
-                for(itr=FileList.begin();itr!=FileList.end();itr++)
+                for(itr=FileList.begin();itr!=FileList.end();++itr)
                   {
                     InFilename = *itr;
                     ifstream ifs;
@@ -1008,11 +1180,18 @@ namespace OpenBabel {
                       {
                         //Batch conversion
                         string batchfile = BatchFileName(OutputFileName,*itr);
+                        
+                        //With inputs like babel test.xxx -oyyy -m
+                        //the user may have wanted to do a splitting operation
+                        //Issue a message and abort if xxx==yyy which would overwrite input file
+                        if(FileList.size()==1 && !CheckForUnintendedBatch(batchfile, InFilename))
+                          return Count;
+
                         if(ofs.is_open()) ofs.close();
                         ofs.open(batchfile.c_str(), omode);
                         if(!ofs) 
                           {
-                            cerr << "Cannot open " << batchfile << endl;
+                            obErrorLog.ThrowError(__FUNCTION__,"Cannot open " + batchfile, obError);
                             return Count;
                           }
                         OutputFileList.push_back(batchfile);
@@ -1026,6 +1205,18 @@ namespace OpenBabel {
                         Count = Convert(&ifs,pOs);					
                       }
                   }
+
+                if(!os.is_open() && !OutputFileName.empty() && !HasMultipleOutputFiles)
+                  {
+                    //Output was written to temporary string stream. Output it to the file
+                    os.open(OutputFileName.c_str(),omode);
+                    if(!os)
+                      {
+                        obErrorLog.ThrowError(__FUNCTION__,"Cannot write to " + OutputFileName, obError);
+                        return Count;
+                      }
+                    os << ssOut.rdbuf();
+                  }
                 return Count;
               }
             else
@@ -1054,8 +1245,10 @@ namespace OpenBabel {
                         SetOneObjectOnly();
 
 #ifdef HAVE_LIBZ
-                        if(Indx==1 && zIn.is_gzip())
+                        if(Indx==1 && zIn.is_gzip()) {
                           SetInStream(&zIn);
+                          CheckedForGzip = true; // we know this one is gzip'ed
+                        }
 #endif
 
                         int ThisFileCount = Convert();
@@ -1067,7 +1260,7 @@ namespace OpenBabel {
                         ofs.open(incrfile.c_str(), omode);
                         if(!ofs)
                           {
-                            cerr << "Cannot write to " << incrfile << endl;
+                            obErrorLog.ThrowError(__FUNCTION__,"Cannot write to " + incrfile, obError);
                             return Count;
                           }
 						
@@ -1094,13 +1287,26 @@ namespace OpenBabel {
 
         //Single input and output files
         Count = Convert(pIs,pOs);
+
+        if(!os.is_open() && !OutputFileName.empty())
+          {
+            //Output was written to temporary string stream. Output it to the file
+            os.open(OutputFileName.c_str(),omode);
+            if(!os)
+              {
+                obErrorLog.ThrowError(__FUNCTION__,"Cannot write to " + OutputFileName, obError);
+                return Count;
+              }
+            os << ssOut.rdbuf();
+          }
         return Count;
       }
     catch(...)
       {
-        cerr << "Conversion failed with an exception. Count=" << Count <<endl;
+        obErrorLog.ThrowError(__FUNCTION__, "Conversion failed with an exception.",obError);
         return Count;
       }
+    return Count;
   }
 
   bool OBConversion::OpenAndSetFormat(bool SetFormat, ifstream* is)
@@ -1115,8 +1321,8 @@ namespace OpenBabel {
             string ext;
             if(pos!=string::npos)
               ext = InFilename.substr(pos);
-            cerr << "Cannot read input format \"" << ext << '\"' 
-                 << " for file \"" << InFilename << "\"" << endl;
+            obErrorLog.ThrowError(__FUNCTION__, "Cannot read input format \"" 
+                                  + ext + '\"' + " for file \"" + InFilename + "\"",obError);
             return false;
           }
       }
@@ -1131,7 +1337,7 @@ namespace OpenBabel {
     is->open(InFilename.c_str(), imode);
     if(!is->good())
       {
-        cerr << "Cannot open " << InFilename <<endl;
+        obErrorLog.ThrowError(__FUNCTION__, "Cannot open " + InFilename, obError);
         return false;
       }
 
@@ -1183,12 +1389,9 @@ namespace OpenBabel {
       }
   }
 
-  typedef std::map<string,int> OPAMapType;
-  OPAMapType& OBConversion::OptionParamArray(Option_type typ)
+  OBConversion::OPAMapType& OBConversion::OptionParamArray(Option_type typ)
   {
-    static OPAMapType* opa = NULL;
-    if (!opa)
-      opa = new OPAMapType[3];
+    static OPAMapType* opa = new OPAMapType[3];
     return opa[typ];
   }
 
@@ -1205,9 +1408,10 @@ namespace OpenBabel {
             string description("API");
             if(pFormat)
               description=pFormat->Description();
-            cerr << "The number of parameters needed by option \"" << name << "\" in " 
-                 << description.substr(0,description.find('\n'))
-                 << " differs from an earlier registration." << endl;
+            obErrorLog.ThrowError(__FUNCTION__, 
+                                  "The number of parameters needed by option \"" + name + "\" in " 
+                                  + description.substr(0,description.find('\n'))
+                                  + " differs from an earlier registration.", obError);
             return;
           }
       }
@@ -1224,6 +1428,49 @@ namespace OpenBabel {
     return pos->second;
   }
 
+  /**
+   * Returns the list of supported input format
+   */
+  std::vector<std::string> OBConversion::GetSupportedInputFormat()
+  {
+    if( SupportedInputFormat.empty() )
+    {
+      Formatpos pos;
+      OBFormat *pFormat;
+      const char* str = NULL;
+      while( GetNextFormat(pos, str, pFormat) )
+      {
+        if( (pFormat->Flags() & NOTREADABLE) )
+        {
+          continue;
+        }
+        SupportedInputFormat.push_back(str);
+      }
+    }
+    return( SupportedInputFormat );
+  }
+  /**
+   * Returns the list of supported output format
+   */
+  std::vector<std::string> OBConversion::GetSupportedOutputFormat()
+  {
+    if( SupportedOutputFormat.empty() )
+    {
+      Formatpos pos;
+      OBFormat *pFormat;
+      const char* str = NULL;
+      while( GetNextFormat(pos, str, pFormat) )
+      {
+        if( ( pFormat->Flags() & NOTWRITABLE) )
+        {
+          continue;
+        }
+        SupportedOutputFormat.push_back(str);
+      }
+    }
+    return( SupportedOutputFormat );
+  }
+
 }//namespace OpenBabel
 
 //! \file obconversion.cpp
