@@ -2,7 +2,7 @@
 obutil.cpp - Various utility methods.
  
 Copyright (C) 1998-2001 by OpenEye Scientific Software, Inc.
-Some portions Copyright (C) 2001-2005 by Geoffrey R. Hutchison
+Some portions Copyright (C) 2001-2006 by Geoffrey R. Hutchison
  
 This file is part of the Open Babel project.
 For more information, see <http://openbabel.sourceforge.net/>
@@ -17,11 +17,10 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 ***********************************************************************/
 
-#include "babelconfig.h"
-#include "math/matrix3x3.h"
-#include "math/vector3.h"
-#include "mol.h"
-#include "obutil.h"
+#include <openbabel/babelconfig.h>
+#include <openbabel/math/matrix3x3.h>
+#include <openbabel/mol.h>
+#include <openbabel/obutil.h>
 
 #if HAVE_CONIO_H
 #include <conio.h>
@@ -31,7 +30,7 @@ using namespace std;
 namespace OpenBabel
 {
 
-  /*! \class OBStopwatch
+  /*! \class OBStopwatch obutil.h <openbabel/obutil.h>
     \brief Stopwatch class used for timing length of execution
 
     The OBStopwatch class makes timing the execution of blocks of
@@ -60,7 +59,7 @@ namespace OpenBabel
     obErrorLog.ThrowError("", str, obInfo);
   }
 
-  // Comparison function (for sorting ints) returns a < b
+  // returns True if a < b, False otherwise.
   OBAPI bool OBCompareInt(const int &a,const int &b)
   {
     return(a<b);
@@ -72,22 +71,39 @@ namespace OpenBabel
     return(a<b);
   }
 
-  // Comparison for doubles: returns a < (b + epsilon)
+  //! Comparison for doubles: returns fabs(a - b) < epsilon
   OBAPI bool IsNear(const double &a, const double &b, const double epsilon)
   {
     return (fabs(a - b) < epsilon);
   }
 
-  // Comparison for doubles: returns a < (0.0 + epsilon)
+  //! Comparison for doubles: returns fabs(a) < epsilon
   OBAPI bool IsNearZero(const double &a, const double epsilon)
   {
     return (fabs(a) < epsilon);
   }
+  
+  //! Comparison for nan (not a number)
+  OBAPI bool IsNan(const double &a)
+  {
+    return ((a) != (a));
+  }
+
+  //! Tests whether its argument can be squared without triggering an overflow or
+  //! underflow.
+  OBAPI bool CanBeSquared(const double &a)
+  {
+    if( a == 0 ) return true;
+    const double max_squarable_double = 1e150;
+    const double min_squarable_double = 1e-150;
+    double abs_a = fabs(a);
+    return(abs_a < max_squarable_double && abs_a > min_squarable_double);
+  }
 
   //! Utility function: replace the last extension in string &src with new extension char *ext.
   OBAPI string NewExtension(string &src,char *ext)
   {
-    unsigned int pos = (unsigned int)src.find_last_of(".");
+    string::size_type pos = (unsigned int)src.find_last_of(".");
     string dst;
 
     if (pos != string::npos)
@@ -102,19 +118,18 @@ namespace OpenBabel
     return(dst);
   }
 
-  //! Return the geometric centroid to an array of coordinates in double* format
+  //! \return the geometric centroid to an array of coordinates in double* format
   //!  and center the coordinates to the origin. Operates on the first "size" 
   //!  coordinates in the array.
   OBAPI vector3 center_coords(double *c, unsigned int size)
   {
     if (size == 0)
       {
-        vector3 v(0.0f, 0.0f, 0.0f);
-        return(v);
+        return(VZero);
       }
 		unsigned int i; 
-    double x=0,y=0,z=0;
-    for (i = 0;i < size;i++)
+    double x=0.0, y=0.0, z=0.0;
+    for (i = 0;i < size;++i)
       {
         x += c[i*3];
         y += c[i*3+1];
@@ -123,7 +138,7 @@ namespace OpenBabel
     x /= (double) size;
     y /= (double) size;
     z /= (double) size;
-    for (i = 0;i < size;i++)
+    for (i = 0;i < size;++i)
       {
         c[i*3]   -= x;
         c[i*3+1] -= y;
@@ -138,7 +153,7 @@ namespace OpenBabel
   OBAPI void rotate_coords(double *c,double m[3][3],unsigned int size)
   {
     double x,y,z;
-    for (unsigned int i = 0;i < size;i++)
+    for (unsigned int i = 0;i < size;++i)
       {
         x = c[i*3]*m[0][0] + c[i*3+1]*m[0][1] + c[i*3+2]*m[0][2];
         y = c[i*3]*m[1][0] + c[i*3+1]*m[1][1] + c[i*3+2]*m[1][2];
@@ -153,10 +168,10 @@ namespace OpenBabel
   OBAPI double calc_rms(double *r,double *f, unsigned int N)
   {
     if (N == 0)
-      return 0.0f; // no RMS deviation between two empty sets
+      return 0.0; // no RMS deviation between two empty sets
 
     double d2=0.0;
-    for (unsigned int i = 0;i < N;i++)
+    for (unsigned int i = 0;i < N;++i)
       {
         d2 += SQUARE(r[i*3] - f[i*3]) +
           SQUARE(r[i*3+1] - f[i*3+1]) +
@@ -252,7 +267,7 @@ namespace OpenBabel
     tz = c[tor[1]+2];
     vector<int>::iterator i;
     int j;
-    for (i = atoms.begin();i != atoms.end();i++)
+    for (i = atoms.begin();i != atoms.end();++i)
       {
         j = *i;
         c[j] -= tx;
@@ -272,10 +287,10 @@ namespace OpenBabel
 
   //! Safely open the supplied filename and return an ifstream, throwing an error
   //! to the default OBMessageHandler error log if it fails.
-  OBAPI bool SafeOpen(ifstream &fs,char *filename)
+  OBAPI bool SafeOpen(ifstream &fs, const char *filename)
   {
 #ifdef WIN32
-    string s = filename;
+    string s(filename);
     if (s.find(".bin") != string::npos)
       fs.open(filename,ios::binary);
     else
@@ -298,17 +313,17 @@ namespace OpenBabel
 
   //! Safely open the supplied filename and return an ofstream, throwing an error
   //! to the default OBMessageHandler error log if it fails.
-  OBAPI bool SafeOpen(ofstream &fs,char *filename)
+  OBAPI bool SafeOpen(ofstream &fs, const char *filename)
   {
 #ifdef WIN32
-    string s = filename;
+    string s(filename);
     if (s.find(".bin") != string::npos)
       fs.open(filename,ios::binary);
     else
 #endif
 
       fs.open(filename);
-
+    
     if (!fs)
       {
         string error = "Unable to open file \'";
@@ -317,22 +332,22 @@ namespace OpenBabel
         obErrorLog.ThrowError(__FUNCTION__, error, obError);
         return(false);
       }
-
+    
     return(true);
   }
 
   //! Safely open the supplied filename and return an ifstream, throwing an error
   //! to the default OBMessageHandler error log if it fails.
-  OBAPI bool SafeOpen(ifstream &fs,string &filename)
+  OBAPI bool SafeOpen(ifstream &fs, const string &filename)
   {
-    return(SafeOpen(fs,(char*)filename.c_str()));
+    return(SafeOpen(fs, filename.c_str()));
   }
 
   //! Safely open the supplied filename and return an ofstream, throwing an error
   //! to the default OBMessageHandler error log if it fails.
-  OBAPI bool SafeOpen(ofstream &fs,string &filename)
+  OBAPI bool SafeOpen(ofstream &fs, const string &filename)
   {
-    return(SafeOpen(fs,(char*)filename.c_str()));
+    return(SafeOpen(fs, filename.c_str()));
   }
 
   //! Shift the supplied string to uppercase
@@ -341,7 +356,7 @@ namespace OpenBabel
     if (s.empty())
       return;
     unsigned int i;
-    for (i = 0;i < s.size();i++)
+    for (i = 0;i < s.size();++i)
       if (isalpha(s[i]) && !isdigit(s[i]))
         s[i] = toupper(s[i]);
   }
@@ -350,7 +365,7 @@ namespace OpenBabel
   OBAPI void ToUpper(char *cptr)
   {
     char *c;
-    for (c = cptr;*c != '\0';c++)
+    for (c = cptr;*c != '\0';++c)
       if (isalpha(*c) && !isdigit(*c))
         *c = toupper(*c);
   }
@@ -361,7 +376,7 @@ namespace OpenBabel
     if (s.empty())
       return;
     unsigned int i;
-    for (i = 0;i < s.size();i++)
+    for (i = 0;i < s.size();++i)
       if (isalpha(s[i]) && !isdigit(s[i]))
         s[i] = tolower(s[i]);
   }
@@ -370,11 +385,37 @@ namespace OpenBabel
   OBAPI void ToLower(char *cptr)
   {
     char *c;
-    for (c = cptr;*c != '\0';c++)
+    for (c = cptr;*c != '\0';++c)
       if (isalpha(*c) && !isdigit(*c))
         *c = tolower(*c);
   }
 
+  //! Shift the supplied string: lowercase to upper, and upper to lower
+  //! \param s - The string to switch case
+  //! \param start - The position to start inverting case
+  OBAPI void InvertCase(std::string &s, unsigned int start)
+  {
+    if (s.empty() || start >= s.size())
+      return;
+    unsigned int i;
+    for (i = start; i < s.size();++i)
+      if (isalpha(s[i]) && !isdigit(s[i])) {
+        if (isupper(s[i])) s[i] = tolower(s[i]);
+        else s[i] = toupper(s[i]);
+      }
+  }
+
+  //! Shift the supplied char*: lowercase to upper, and upper to lower
+  OBAPI void InvertCase(char *cptr)
+  {
+    char *c;
+    for (c = cptr;*c != '\0';++c)
+      if (isalpha(*c) && !isdigit(*c)) {
+        if (isupper(*c)) *c = tolower(*c);
+        else *c = toupper(*c);
+      }
+  }
+
   //! "Clean" the supplied atom type, shifting the first character to uppercase,
   //! the second character (if it's a letter) to lowercase, and terminating with a NULL
   //! to strip off any trailing characters
@@ -398,8 +439,8 @@ namespace OpenBabel
     vector3 n,nn,v1,v2,v3,avec,bvec,cvec;
     double dst = 0.0, ang = 0.0, tor = 0.0;
     OBAtom *atom;
-    vector<OBNodeBase*>::iterator i;
-    int index;
+    vector<OBAtom*>::iterator i;
+    unsigned int index;
 
     if (vic.empty())
       return;
@@ -448,7 +489,7 @@ namespace OpenBabel
           {
             // atom 3
             cvec = VY;
-            tor = 90.0 * DEG_TO_RAD;
+            tor = 90. * DEG_TO_RAD;
           }
 
         v1 = avec - bvec;
@@ -484,7 +525,7 @@ namespace OpenBabel
   {
     double r,sum;
     OBAtom *atom,*nbr,*ref;
-    vector<OBNodeBase*>::iterator i,j,m;
+    vector<OBAtom*>::iterator i,j,m;
 
     obErrorLog.ThrowError(__FUNCTION__,
                           "Ran OpenBabel::CartesianToInternal", obAuditMsg);
@@ -552,25 +593,22 @@ namespace OpenBabel
     unsigned int k;
     vector3 v1,v2;
     OBAtom *a,*b,*c;
-    for (k = 2;k <= mol.NumAtoms();k++)
+    for (k = 2;k <= mol.NumAtoms();++k)
       {
         atom = mol.GetAtom(k);
         a = vic[k]->_a;
         b = vic[k]->_b;
         c = vic[k]->_c;
-        if (k == 2)
-          {
-            vic[k]->_dst = (atom->GetVector() - a->GetVector()).length();
-            continue;
-          }
-
         v1 = atom->GetVector() - a->GetVector();
-        v2 = b->GetVector()    - a->GetVector();
         vic[k]->_dst = v1.length();
-        vic[k]->_ang = vectorAngle(v1,v2);
+        if (k == 2)
+          continue;
 
+        v2 = b->GetVector()    - a->GetVector();
+        vic[k]->_ang = vectorAngle(v1,v2);
         if (k == 3)
           continue;
+
         vic[k]->_tor = CalcTorsionAngle(atom->GetVector(),
                                         a->GetVector(),
                                         b->GetVector(),
@@ -580,7 +618,7 @@ namespace OpenBabel
     //check for linear geometries and try to correct if possible
     bool done;
     double ang;
-    for (k = 2;k <= mol.NumAtoms();k++)
+    for (k = 2;k <= mol.NumAtoms();++k)
       {
         ang = fabs(vic[k]->_ang);
         if (ang > 5.0 && ang < 175.0)
@@ -638,7 +676,7 @@ namespace OpenBabel
     xzyy = 0.0;
     xzyz = 0.0;
 
-    for (i = 0; i < size; i++)
+    for (i = 0; i < size; ++i)
       {
         rx = r[i*3];
         ry = r[i*3+1];
@@ -711,14 +749,12 @@ namespace OpenBabel
 #define FourThirdsPI  (4.0*PI/3.0)
 #define TwoThirdsPI   (2.0*PI/3.0)
 
-#ifdef OLD_RMAT
-
   /*FUNCTION */
   /* recieves: the co-efficients for a general
    *           equation of degree one.
    *           Ax + B = 0 !!
    */
-  OBAPI static int SolveLinear(double A,double B)
+  OBAPI int SolveLinear(double A,double B)
   {
     if( IsZero(A) )
       return( 0 );
@@ -731,7 +767,7 @@ namespace OpenBabel
    *           linear equation of degree two.
    *           Ax^2 + Bx + C = 0 !!
    */
-  OBAPI static int SolveQuadratic(double A,double B,double C)
+  OBAPI int SolveQuadratic(double A,double B,double C)
   {
     register double Descr, Temp, TwoA;
 
@@ -747,11 +783,6 @@ namespace OpenBabel
     if( Descr>0.0 )
       {
         Descr = sqrt(Descr);
-#ifdef ORIG
-
-        Roots[0] = (-B-Descr)/TwoA;
-        Roots[1] = (-B+Descr)/TwoA;
-#else
         /* W. Press, B. Flannery, S. Teukolsky and W. Vetterling,
          * "Quadratic and Cubic Equations", Numerical Recipes in C,
          * Chapter 5, pp. 156-157, 1989.
@@ -759,7 +790,6 @@ namespace OpenBabel
         Temp = (B<0.0)? -0.5*(B-Descr) : -0.5*(B+Descr);
         Roots[0] = Temp/A;
         Roots[1] = C/Temp;
-#endif
 
         return( 2 );
       }
@@ -772,7 +802,7 @@ namespace OpenBabel
    *       given value taking into account
    *       that it may be negative.
    */
-  OBAPI static double CubeRoot(double X)
+  OBAPI double CubeRoot(double X)
   {
     if( X>=0.0 )
       {
@@ -782,7 +812,7 @@ namespace OpenBabel
       return -pow( -X, OneThird );
   }
 
-  OBAPI static int SolveCubic(double A,double B,double C,double D)
+  OBAPI int SolveCubic(double A,double B,double C,double D)
   {
     register double TwoA, ThreeA, BOver3A;
     register double Temp, POver3, QOver2;
@@ -830,7 +860,6 @@ namespace OpenBabel
 
     return( 1 );
   }
-#endif
 
 
 #define MAX_SWEEPS 50
@@ -845,23 +874,23 @@ namespace OpenBabel
 
     memset((char*)d,'\0',sizeof(double)*3);
 
-    for (j = 0; j < 3; j++)
+    for (j = 0; j < 3; ++j)
       {
-        for (i = 0; i < 3; i++)
+        for (i = 0; i < 3; ++i)
           v[i][j] = 0.0;
 
         v[j][j] = 1.0;
         d[j] = a[j][j];
       }
 
-    for (l = 1; l <= MAX_SWEEPS; l++)
+    for (l = 1; l <= MAX_SWEEPS; ++l)
       {
         dnorm = 0.0;
         onorm = 0.0;
-        for (j = 0; j < 3; j++)
+        for (j = 0; j < 3; ++j)
           {
             dnorm = dnorm + (double)fabs(d[j]);
-            for (i = 0; i <= j - 1; i++)
+            for (i = 0; i <= j - 1; ++i)
               {
                 onorm = onorm + (double)fabs(a[i][j]);
               }
@@ -869,9 +898,9 @@ namespace OpenBabel
 
         if((onorm/dnorm) <= 1.0e-12)
           goto Exit_now;
-        for (j = 1; j < 3; j++)
+        for (j = 1; j < 3; ++j)
           {
-            for (i = 0; i <= j - 1; i++)
+            for (i = 0; i <= j - 1; ++i)
               {
                 b = a[i][j];
                 if(fabs(b) > 0.0)
@@ -889,25 +918,25 @@ namespace OpenBabel
                     c = 1.0/(double)sqrt(t * t + 1.0);
                     s = t * c;
                     a[i][j] = 0.0;
-                    for (k = 0; k <= i-1; k++)
+                    for (k = 0; k <= i-1; ++k)
                       {
                         atemp = c * a[k][i] - s * a[k][j];
                         a[k][j] = s * a[k][i] + c * a[k][j];
                         a[k][i] = atemp;
                       }
-                    for (k = i+1; k <= j-1; k++)
+                    for (k = i+1; k <= j-1; ++k)
                       {
                         atemp = c * a[i][k] - s * a[k][j];
                         a[k][j] = s * a[i][k] + c * a[k][j];
                         a[i][k] = atemp;
                       }
-                    for (k = j+1; k < 3; k++)
+                    for (k = j+1; k < 3; ++k)
                       {
                         atemp = c * a[i][k] - s * a[j][k];
                         a[j][k] = s * a[i][k] + c * a[j][k];
                         a[i][k] = atemp;
                       }
-                    for (k = 0; k < 3; k++)
+                    for (k = 0; k < 3; ++k)
                       {
                         vtemp = c * v[k][i] - s * v[k][j];
                         v[k][j] = s * v[k][i] + c * v[k][j];
@@ -925,11 +954,11 @@ namespace OpenBabel
 
     /* max_sweeps = l;*/
 
-    for (j = 0; j < 3-1; j++)
+    for (j = 0; j < 3-1; ++j)
       {
         k = j;
         dtemp = d[k];
-        for (i = j+1; i < 3; i++)
+        for (i = j+1; i < 3; ++i)
           if(d[i] < dtemp)
             {
               k = i;
@@ -940,7 +969,7 @@ namespace OpenBabel
           {
             d[k] = d[j];
             d[j] = dtemp;
-            for (i = 0; i < 3 ; i++)
+            for (i = 0; i < 3 ; ++i)
               {
                 dtemp = v[i][k];
                 v[i][k] = v[i][j];
@@ -1021,11 +1050,11 @@ namespace OpenBabel
     double mat[3][3],rmat[3][3],mat2[3][3],roots[3];
 
     /* make inertial cross tensor */
-    for(i=0;i<3;i++)
-      for(j=0;j<3;j++)
+    for(i=0;i<3;++i)
+      for(j=0;j<3;++j)
         mat[i][j]=0.0;
 
-    for(i=0;i < size;i++)
+    for(i=0;i < size;++i)
       {
         mat[0][0]+=r[3*i]  *f[3*i];
         mat[1][0]+=r[3*i+1]*f[3*i];
@@ -1044,8 +1073,8 @@ namespace OpenBabel
 
 
     /* square matrix= ((mat transpose) * mat) */
-    for(i=0;i<3;i++)
-      for(j=0;j<3;j++)
+    for(i=0;i<3;++i)
+      for(j=0;j<3;++j)
         {
           x=mat[0][i]*mat[0][j]+mat[1][i]*mat[1][j]+mat[2][i]*mat[2][j];
           mat2[i][j]=mat[i][j];
@@ -1073,22 +1102,22 @@ namespace OpenBabel
           roots[2]*=-1.0;
       }
 
-    for(i=0;i<3;i++)
-      for(j=0;j<3;j++)
+    for(i=0;i<3;++i)
+      for(j=0;j<3;++j)
         mat[i][j]=roots[0]*rmat[i][0]*rmat[j][0]+
           roots[1]*rmat[i][1]*rmat[j][1]+
           roots[2]*rmat[i][2]*rmat[j][2];
 
     /* and multiply into original inertial cross matrix, mat2 */
-    for(i=0;i<3;i++)
-      for(j=0;j<3;j++)
+    for(i=0;i<3;++i)
+      for(j=0;j<3;++j)
         rmat[i][j]=mat[0][j]*mat2[i][0]+
           mat[1][j]*mat2[i][1]+
           mat[2][j]*mat2[i][2];
 
     /* rotate all coordinates */
     d2 = 0.0;
-    for(i=0;i<size;i++)
+    for(i=0;i<size;++i)
       {
         x=f[3*i]*rmat[0][0]+f[3*i+1]*rmat[0][1]+f[3*i+2]*rmat[0][2];
         y=f[3*i]*rmat[1][0]+f[3*i+1]*rmat[1][1]+f[3*i+2]*rmat[1][2];
@@ -1115,11 +1144,11 @@ namespace OpenBabel
     double mat[3][3],rmat[3][3],mat2[3][3],roots[3];
 
     /* make inertial cross tensor */
-    for(i=0;i<3;i++)
-      for(j=0;j<3;j++)
+    for(i=0;i<3;++i)
+      for(j=0;j<3;++j)
         mat[i][j]=0.0;
 
-    for(i=0;i < size;i++)
+    for(i=0;i < size;++i)
       {
         mat[0][0]+=r[3*i]  *f[3*i];
         mat[1][0]+=r[3*i+1]*f[3*i];
@@ -1137,8 +1166,8 @@ namespace OpenBabel
       +mat[0][2]*(mat[1][0]*mat[2][1]-mat[1][1]*mat[2][0]);
 
     /* square matrix= ((mat transpose) * mat) */
-    for(i=0;i<3;i++)
-      for(j=0;j<3;j++)
+    for(i=0;i<3;++i)
+      for(j=0;j<3;++j)
         {
           x=mat[0][i]*mat[0][j]+mat[1][i]*mat[1][j]+mat[2][i]*mat[2][j];
           mat2[i][j]=mat[i][j];
@@ -1166,15 +1195,15 @@ namespace OpenBabel
           roots[2]*=-1.0;
       }
 
-    for(i=0;i<3;i++)
-      for(j=0;j<3;j++)
+    for(i=0;i<3;++i)
+      for(j=0;j<3;++j)
         mat[i][j]=roots[0]*rmat[i][0]*rmat[j][0]+
           roots[1]*rmat[i][1]*rmat[j][1]+
           roots[2]*rmat[i][2]*rmat[j][2];
 
     /* and multiply into original inertial cross matrix, mat2 */
-    for(i=0;i<3;i++)
-      for(j=0;j<3;j++)
+    for(i=0;i<3;++i)
+      for(j=0;j<3;++j)
         rmat[i][j]=mat[0][j]*mat2[i][0]+
           mat[1][j]*mat2[i][1]+
           mat[2][j]*mat2[i][2];
